@@ -13,14 +13,12 @@
 #include <crispy/StackTrace.h>
 #include <crispy/utils.h>
 
-#include <fmt/chrono.h>
-#include <fmt/format.h>
-
 #include <QtCore/QFile>
 
 #include <chrono>
 #include <csignal>
 #include <cstdio>
+#include <format>
 #include <fstream>
 #include <iostream>
 #include <memory>
@@ -81,7 +79,7 @@ namespace
         crashLogger(sstr);
         string const crashLog = sstr.str();
 
-        auto const logFileName = fmt::format(
+        auto const logFileName = std::format(
             "contour-crash-{:%Y-%m-%d-%H-%M-%S}-pid-{}.log", std::chrono::system_clock::now(), getpid());
         if (chdir(crashLogDir.c_str()) < 0)
             perror("chdir");
@@ -160,6 +158,8 @@ ContourApp::ContourApp(): app("contour", "Contour Terminal Emulator", CONTOUR_VE
     link("contour.generate.config", bind(&ContourApp::configAction, this));
     link("contour.generate.integration", bind(&ContourApp::integrationAction, this));
     link("contour.info.vt", bind(&ContourApp::infoVT, this));
+    link("contour.documentation.vt", bind(&ContourApp::documentationVT, this));
+    link("contour.documentation.keys", bind(&ContourApp::documentationKeyMapping, this));
 }
 
 template <typename Callback>
@@ -178,6 +178,74 @@ auto withOutput(crispy::cli::flag_store const& flags, std::string const& name, C
     return callback(*out);
 }
 
+int ContourApp::documentationVT()
+{
+    using category = vtbackend::FunctionCategory;
+    using namespace std::string_view_literals;
+
+    std::string info;
+    auto back = std::back_inserter(info);
+    std::format_to(back, "# {}\n", "VT sequences");
+    std::format_to(back, "{}\n\n", "List of VT sequences supported by Contour Terminal Emulator.");
+    for (auto const& [category, headline]: { std::pair { category::C0, "Control Codes"sv },
+                                             std::pair { category::ESC, "Escape Sequences"sv },
+                                             std::pair { category::CSI, "Control Sequences"sv },
+                                             std::pair { category::OSC, "Operating System Commands"sv },
+                                             std::pair { category::DCS, "Device Control Sequences"sv } })
+    {
+
+        std::format_to(back, "## {}\n\n", headline);
+
+        std::format_to(back, "| Sequence | Code | Description |\n");
+        std::format_to(back, "|----------|------|-------------|\n");
+        for (auto const& fn: vtbackend::allFunctions())
+        {
+            if (fn.category != category)
+                continue;
+
+            // This could be much more improved in good looking and informationally.
+            // We can also print short/longer description, minimum required VT level,
+            // colored output for easier reading, and maybe more.
+            std::format_to(back,
+                           "| `{:}` | {:} | {:} |\n",
+                           crispy::escapeMarkdown(std::format("{}", fn)),
+                           fn.documentation.mnemonic,
+                           fn.documentation.comment);
+        }
+        std::format_to(back, "\n");
+    }
+
+    std::cout << info;
+    return EXIT_SUCCESS;
+}
+
+int ContourApp::documentationKeyMapping()
+{
+    std::string info;
+    auto back = std::back_inserter(info);
+    std::format_to(back, "{}\n\n", "List of supported actions for key mappings.");
+
+    std::format_to(back, "| Action | Description |\n");
+    std::format_to(back, "|--------|-------------|\n");
+    for (auto const& [action, description]: contour::actions::getDocumentation())
+    {
+        std::format_to(back, "| `{:<20}` | {:} |\n", action, description);
+    }
+
+    std::format_to(back, "\n");
+    std::format_to(back, "Example of entries inside config file\n");
+    std::format_to(back, "``` yaml\n");
+    for (auto const& [action, description]: contour::actions::getDocumentation())
+    {
+        std::format_to(back, " - {{ mods: [Control], key: Enter, action: {:} }}\n", action);
+    }
+    std::format_to(back, "```\n");
+    std::format_to(back, "\n");
+
+    std::cout << info;
+    return EXIT_SUCCESS;
+}
+
 int ContourApp::infoVT()
 {
     using category = vtbackend::FunctionCategory;
@@ -191,27 +259,27 @@ int ContourApp::infoVT()
                                              pair { category::OSC, "Operating System Commands"sv },
                                              pair { category::DCS, "Device Control Sequences"sv } })
     {
-        fmt::print("{}\n", headline);
-        fmt::print("{}\n\n", string(headline.size(), '='));
+        std::cout << std::format("{}\n", headline);
+        std::cout << std::format("{}\n\n", string(headline.size(), '='));
 
         for (auto const& fn: vtbackend::allFunctions())
         {
             if (fn.category != category)
                 continue;
 
-            auto const level = fn.extension == VTExtension::None ? fmt::format("{}", fn.conformanceLevel)
-                                                                 : fmt::format("{}", fn.extension);
+            auto const level = fn.extension == VTExtension::None ? std::format("{}", fn.conformanceLevel)
+                                                                 : std::format("{}", fn.extension);
 
             // This could be much more improved in good looking and informationally.
             // We can also print short/longer description, minimum required VT level,
             // colored output for easier reading, and maybe more.
-            fmt::print("{:<20} {:<15} {} ({})\n",
-                       fn.documentation.mnemonic,
-                       fmt::format("{}", fn),
-                       fn.documentation.comment,
-                       level);
+            std::cout << std::format("{:<20} {:<15} {} ({})\n",
+                                     fn.documentation.mnemonic,
+                                     std::format("{}", fn),
+                                     fn.documentation.comment,
+                                     level);
         }
-        fmt::print("\n");
+        std::cout << std::format("\n");
     }
 
     return EXIT_SUCCESS;
@@ -228,9 +296,11 @@ int ContourApp::integrationAction()
             file.setFileName(":/contour/shell-integration/shell-integration.fish");
         else if (shell == "tcsh")
             file.setFileName(":/contour/shell-integration/shell-integration.tcsh");
+        else if (shell == "bash")
+            file.setFileName(":/contour/shell-integration/shell-integration.bash");
         else
         {
-            std::cerr << fmt::format("Cannot generate shell integration for an unsupported shell, {}.\n",
+            std::cerr << std::format("Cannot generate shell integration for an unsupported shell, {}.\n",
                                      shell);
             return EXIT_FAILURE;
         }
@@ -290,7 +360,7 @@ int ContourApp::profileAction()
 {
     auto const profileName = parameters().get<string>("contour.set.profile.to");
     // TODO: guard `profileName` value against invalid input.
-    cout << fmt::format("\033P$p{}\033\\", profileName);
+    cout << std::format("\033P$p{}\033\\", profileName);
     return EXIT_SUCCESS;
 }
 
@@ -313,7 +383,15 @@ crispy::cli::command ContourApp::parameterDefinition() const
                 CLI::option_list {},
                 CLI::command_list {
                     CLI::command { "vt", "Prints general information about supported VT sequences." },
+                    CLI::command { "config", "Prints missing entries from user config file." },
                 } },
+            CLI::command { "documentation",
+                           "Generate documentation for web page",
+                           CLI::option_list {},
+                           CLI::command_list {
+                               CLI::command { "vt", "VT sequence reference documentation" },
+                               CLI::command { "keys", "List of configurable actions for key binding" },
+                           } },
             CLI::command {
                 "generate",
                 "Generation utilities.",
@@ -323,7 +401,8 @@ crispy::cli::command ContourApp::parameterDefinition() const
                                    "Dumps VT parser's state machine in dot-file format to stdout." },
                     CLI::command {
                         "terminfo",
-                        "Generates the terminfo source file that will reflect the features of this version "
+                        "Generates the terminfo source file that will reflect the features of "
+                        "this version "
                         "of contour. Using - as value will write to stdout instead.",
                         {
                             CLI::option { "to",
@@ -387,14 +466,15 @@ crispy::cli::command ContourApp::parameterDefinition() const
                 "set",
                 "Sets various aspects of the connected terminal.",
                 CLI::option_list {},
-                CLI::command_list { CLI::command {
-                    "profile",
-                    "Changes the terminal profile of the currently attached terminal to the given value.",
-                    CLI::option_list {
-                        CLI::option { "to",
-                                      CLI::value { ""s },
-                                      "Profile name to activate in the currently connected terminal.",
-                                      "NAME" } } } } } }
+                CLI::command_list {
+                    CLI::command { "profile",
+                                   "Changes the terminal profile of the currently attached terminal to the "
+                                   "given value.",
+                                   CLI::option_list { CLI::option {
+                                       "to",
+                                       CLI::value { ""s },
+                                       "Profile name to activate in the currently connected terminal.",
+                                       "NAME" } } } } } }
     };
 }
 
