@@ -25,6 +25,8 @@ class ContourApp: public crispy::app
     int configAction();
     int integrationAction();
     int infoVT();
+    int documentationVT();
+    int documentationKeyMapping();
 };
 
 } // namespace contour
