@@ -12,9 +12,11 @@
 #include <crispy/utils.h>
 
 #include <QtCore/QProcess>
+#include <QtQml/qqmlextensionplugin.h>
 #if !defined(__APPLE__) && !defined(_WIN32)
     #include <QtDBus/QDBusConnection>
 #endif
+#include <QtCore/QtPlugin>
 #include <QtDBus/QtDBus>
 #include <QtGui/QGuiApplication>
 #include <QtGui/QStyleHints>
@@ -53,6 +55,7 @@ ContourGuiApp::ContourGuiApp(): _sessionManager(*this)
 {
     link("contour.terminal", bind(&ContourGuiApp::terminalGuiAction, this));
     link("contour.font-locator", bind(&ContourGuiApp::fontConfigAction, this));
+    link("contour.info.config", bind(&ContourGuiApp::checkConfig, this));
 }
 
 int ContourGuiApp::run(int argc, char const* argv[])
@@ -109,7 +112,7 @@ crispy::cli::command ContourGuiApp::parameterDefinition() const
                     "Dumps internal state at exit into the given directory. This is for debugging contour.",
                     "PATH" },
                 CLI::option { "early-exit-threshold",
-                              CLI::value { 6u },
+                              CLI::value { -1 },
                               "If the spawned process exits earlier than the given threshold seconds, an "
                               "error message will be printed and the window not closed immediately." },
                 CLI::option { "working-directory",
@@ -149,7 +152,17 @@ crispy::cli::command ContourGuiApp::parameterDefinition() const
 
 std::chrono::seconds ContourGuiApp::earlyExitThreshold() const
 {
-    return std::chrono::seconds(parameters().get<unsigned>("contour.terminal.early-exit-threshold"));
+    auto const configThreshold = config().earlyExitThreshold.value();
+    auto const parameterThreshold = parameters().get<int>("contour.terminal.early-exit-threshold");
+
+    // default threshold is config::documentation::DefaultEarlyExitThreshold seconds
+    if (parameterThreshold >= 0)
+        return std::chrono::seconds(parameterThreshold);
+
+    if (configThreshold != config::documentation::DefaultEarlyExitThreshold)
+        return std::chrono::seconds(configThreshold);
+
+    return std::chrono::seconds(config::documentation::DefaultEarlyExitThreshold);
 }
 
 string ContourGuiApp::profileName() const
@@ -157,11 +170,11 @@ string ContourGuiApp::profileName() const
     if (auto profile = parameters().get<string>("contour.terminal.profile"); !profile.empty())
         return profile;
 
-    if (!_config.defaultProfileName.empty())
-        return _config.defaultProfileName;
+    if (!_config.defaultProfileName.value().empty())
+        return _config.defaultProfileName.value();
 
-    if (_config.profiles.size() == 1)
-        return _config.profiles.begin()->first;
+    if (_config.profiles.value().size() == 1)
+        return _config.profiles.value().begin()->first;
 
     return ""s;
 }
@@ -199,6 +212,20 @@ QUrl ContourGuiApp::resolveResource(std::string_view path)
     return QUrl("qrc:/contour/" + QString::fromLatin1(path.data(), static_cast<int>(path.size())));
 }
 
+int ContourGuiApp::checkConfig()
+{
+    auto const& flags = parameters();
+    auto const configPath = QString::fromStdString(flags.get<string>("contour.terminal.config"));
+
+    _config = configPath.isEmpty() ? contour::config::loadConfig()
+                                   : contour::config::loadConfigFromFile(configPath.toStdString());
+
+    contour::config::compareEntries(
+        _config, logstore::category("", "Console Logger", logstore::category::state::Enabled));
+
+    return EXIT_SUCCESS;
+}
+
 bool ContourGuiApp::loadConfig(string const& target)
 {
     auto const& flags = parameters();
@@ -220,23 +247,23 @@ bool ContourGuiApp::loadConfig(string const& target)
     _config = configPath.isEmpty() ? contour::config::loadConfig()
                                    : contour::config::loadConfigFromFile(configPath.toStdString());
 
-    _config.live = _config.live || parameters().boolean("contour.terminal.live-config");
+    _config.live.value() = _config.live.value() || parameters().boolean("contour.terminal.live-config");
 
     if (!_config.profile(profileName()))
     {
         auto const s =
-            accumulate(begin(_config.profiles),
-                       end(_config.profiles),
+            accumulate(begin(_config.profiles.value()),
+                       end(_config.profiles.value()),
                        ""s,
                        [](string const& acc, auto const& profile) -> string {
-                           return acc.empty() ? profile.first : fmt::format("{}, {}", acc, profile.first);
+                           return acc.empty() ? profile.first : std::format("{}, {}", acc, profile.first);
                        });
         configLogger(
-            fmt::format("No profile with name '{}' found. Available profiles: {}", profileName(), s));
+            std::format("No profile with name '{}' found. Available profiles: {}", profileName(), s));
     }
 
     if (auto const wd = flags.get<string>("contour.terminal.working-directory"); !wd.empty())
-        _config.profile(profileName())->shell.workingDirectory = fs::path(wd);
+        _config.profile(profileName())->shell.value().workingDirectory = fs::path(wd);
 
     config::TerminalProfile* profile = _config.profile(profileName());
     if (!profile)
@@ -249,7 +276,7 @@ bool ContourGuiApp::loadConfig(string const& target)
     auto exe = flags.get<string>("contour.terminal.execute");
     if (!flags.verbatim.empty() || !exe.empty())
     {
-        auto& shell = profile->shell;
+        auto& shell = profile->shell.value();
         shell.arguments.clear();
         if (!exe.empty())
         {
@@ -259,7 +286,27 @@ bool ContourGuiApp::loadConfig(string const& target)
         }
         else
         {
-            shell.program = flags.verbatim.front();
+            auto frontCommand = flags.verbatim.front();
+
+            // check if this is a file
+            if (fs::exists(frontCommand) && fs::is_regular_file(frontCommand))
+            {
+                // check if this is an executable file
+                if ((fs::status(frontCommand).permissions() & fs::perms::owner_exec) != fs::perms::none)
+                    shell.program = frontCommand;
+                else // find a path to file and open shell in this path
+                    shell.workingDirectory = fs::path(frontCommand).parent_path();
+            }
+            else if (fs::exists(frontCommand) && fs::is_directory(frontCommand))
+            {
+                shell.workingDirectory = fs::path(frontCommand);
+            }
+            else
+            {
+                errorLog()("Do not know what to do with `{}` will use it as a program", frontCommand);
+                shell.program = frontCommand;
+            }
+
             for (size_t i = 1; i < flags.verbatim.size(); ++i)
                 shell.arguments.emplace_back(flags.verbatim.at(i));
         }
@@ -276,16 +323,17 @@ int ContourGuiApp::fontConfigAction()
     if (!loadConfig("font-locator"))
         return EXIT_FAILURE;
 
-    vtrasterizer::FontDescriptions const& fonts = _config.profile(_config.defaultProfileName)->fonts;
+    vtrasterizer::FontDescriptions const& fonts =
+        _config.profile(_config.defaultProfileName.value())->fonts.value();
     text::font_description const& fontDescription = fonts.regular;
     text::font_locator& fontLocator = createFontLocator(fonts.fontLocator);
     text::font_source_list const fontSources = fontLocator.locate(fontDescription);
 
-    fmt::print("Matching fonts using  : {}\n", fonts.fontLocator);
-    fmt::print("Font description      : {}\n", fontDescription);
-    fmt::print("Number of fonts found : {}\n", fontSources.size());
+    std::cout << std::format("Matching fonts using  : {}\n", fonts.fontLocator);
+    std::cout << std::format("Font description      : {}\n", fontDescription);
+    std::cout << std::format("Number of fonts found : {}\n", fontSources.size());
     for (text::font_source const& fontSource: fontSources)
-        fmt::print("  {}\n", fontSource);
+        std::cout << std::format("  {}\n", fontSource);
 
     return EXIT_SUCCESS;
 }
@@ -295,7 +343,7 @@ int ContourGuiApp::terminalGuiAction()
     if (!loadConfig("terminal"))
         return EXIT_FAILURE;
 
-    switch (_config.renderingBackend)
+    switch (_config.renderingBackend.value())
     {
         case config::RenderingBackend::OpenGL:
             QGuiApplication::setAttribute(Qt::AA_UseSoftwareOpenGL, false);
@@ -314,6 +362,10 @@ int ContourGuiApp::terminalGuiAction()
         errorLog()("Could not access configuration profile.");
         return EXIT_FAILURE;
     }
+    auto appName = QString::fromStdString(profile->wmClass.value());
+    QCoreApplication::setApplicationName(appName);
+    QCoreApplication::setOrganizationName("contour");
+    QCoreApplication::setApplicationVersion(CONTOUR_VERSION_STRING);
 
     vector<string> qtArgsStore;
     vector<char const*> qtArgsPtr;
@@ -333,11 +385,11 @@ int ContourGuiApp::terminalGuiAction()
     addQtArgIfSet("contour.terminal.session", "-session");
     if (!addQtArgIfSet("contour.terminal.platform", "-platform"))
     {
-        if (!_config.platformPlugin.empty())
+        if (!_config.platformPlugin.value().empty())
         {
             static constexpr auto PlatformArg = string_view("-platform");
             qtArgsPtr.push_back(PlatformArg.data());
-            qtArgsPtr.push_back(_config.platformPlugin.c_str());
+            qtArgsPtr.push_back(_config.platformPlugin.value().c_str());
         }
     }
 
@@ -403,7 +455,7 @@ int ContourGuiApp::terminalGuiAction()
     // Spawn initial window.
     newWindow();
 
-    if (auto const& bell = config().profile().bell.sound; bell == "off")
+    if (auto const& bell = config().profile().bell.value().sound; bell == "off")
     {
         if (auto* bellAudioOutput = _qmlEngine->rootObjects().first()->findChild<QObject*>("BellAudioOutput");
             bellAudioOutput)
@@ -453,12 +505,12 @@ void ContourGuiApp::setupQCoreApplication()
     auto const* profile = _config.profile(profileName());
     Require(profile);
 
-    auto const defaultAppName = QStringLiteral("contour");
+    auto const defaultAppName = QStringLiteral(CONTOUR_APP_ID);
     auto const defaultOrgDomain = QStringLiteral("contour-terminal.org");
     auto const defaultOrgName = QStringLiteral("contour");
 
     auto const platformName = QGuiApplication::platformName();
-    auto const wmClass = QString::fromStdString(profile->wmClass);
+    auto const wmClass = QString::fromStdString(profile->wmClass.value());
 
     auto const effectiveAppName = [&]() -> QString const& {
         // On X11, we want to set the WM_CLASS property to the configured value.
@@ -474,6 +526,8 @@ void ContourGuiApp::setupQCoreApplication()
     // application id on Wayland when using Qt.
     if (platformName == "wayland" && !wmClass.isEmpty())
         QGuiApplication::setDesktopFileName(wmClass);
+    else
+        QGuiApplication::setDesktopFileName(defaultAppName);
 
     QCoreApplication::setOrganizationDomain(defaultOrgDomain);
     QCoreApplication::setOrganizationName(defaultOrgName);
