@@ -36,7 +36,10 @@
 #include <fstream>
 #include <limits>
 
-#if !defined(_WIN32)
+#if defined(__OpenBSD__)
+    #include <pthread_np.h>
+    #define pthread_setname_np pthread_set_name_np
+#elif !defined(_WIN32)
     #include <pthread.h>
 #endif
 
@@ -63,7 +66,7 @@ namespace
 {
     string unhandledExceptionMessage(string_view const& where, exception const& e)
     {
-        return fmt::format("{}: Unhandled exception caught ({}). {}", where, typeid(e).name(), e.what());
+        return std::format("{}: Unhandled exception caught ({}). {}", where, typeid(e).name(), e.what());
     }
 
     void setThreadName(char const* name)
@@ -126,30 +129,32 @@ namespace
     {
         auto settings = vtbackend::Settings {};
 
-        settings.pageSize = profile.terminalSize;
-        settings.ptyBufferObjectSize = config.ptyBufferObjectSize;
-        settings.ptyReadBufferSize = config.ptyReadBufferSize;
-        settings.maxHistoryLineCount = profile.maxHistoryLineCount;
-        settings.copyLastMarkRangeOffset = profile.copyLastMarkRangeOffset;
-        settings.cursorBlinkInterval = profile.inputModes.insert.cursor.cursorBlinkInterval;
-        settings.cursorShape = profile.inputModes.insert.cursor.cursorShape;
-        settings.cursorDisplay = profile.inputModes.insert.cursor.cursorDisplay;
-        settings.smoothLineScrolling = profile.smoothLineScrolling;
-        settings.wordDelimiters = unicode::from_utf8(config.wordDelimiters);
-        settings.mouseProtocolBypassModifiers = config.bypassMouseProtocolModifiers;
-        settings.maxImageSize = config.maxImageSize;
-        settings.maxImageRegisterCount = config.maxImageColorRegisters;
-        settings.statusDisplayType = profile.initialStatusDisplayType;
-        settings.statusDisplayPosition = profile.statusDisplayPosition;
+        settings.pageSize = profile.terminalSize.value();
+        settings.ptyBufferObjectSize = config.ptyBufferObjectSize.value();
+        settings.ptyReadBufferSize = config.ptyReadBufferSize.value();
+        settings.maxHistoryLineCount = profile.maxHistoryLineCount.value();
+        settings.copyLastMarkRangeOffset = profile.copyLastMarkRangeOffset.value();
+        settings.cursorBlinkInterval = profile.modeInsert.value().cursor.cursorBlinkInterval;
+        settings.cursorShape = profile.modeInsert.value().cursor.cursorShape;
+        settings.cursorDisplay = profile.modeInsert.value().cursor.cursorDisplay;
+        settings.smoothLineScrolling = profile.smoothLineScrolling.value();
+        settings.wordDelimiters = unicode::from_utf8(config.wordDelimiters.value());
+        settings.mouseProtocolBypassModifiers = config.bypassMouseProtocolModifiers.value();
+        settings.maxImageSize = config.maxImageSize.value();
+        settings.maxImageRegisterCount = config.maxImageColorRegisters.value();
+        settings.statusDisplayType = profile.initialStatusDisplayType.value();
+        settings.statusDisplayPosition = profile.statusDisplayPosition.value();
+        settings.indicatorStatusLine.left = profile.indicatorStatusLineLeft.value();
+        settings.indicatorStatusLine.middle = profile.indicatorStatusLineMiddle.value();
+        settings.indicatorStatusLine.right = profile.indicatorStatusLineRight.value();
         settings.syncWindowTitleWithHostWritableStatusDisplay =
-            profile.syncWindowTitleWithHostWritableStatusDisplay;
-        if (auto const* p = preferredColorPalette(profile.colors, colorPreference))
+            profile.syncWindowTitleWithHostWritableStatusDisplay.value();
+        if (auto const* p = preferredColorPalette(profile.colors.value(), colorPreference))
             settings.colorPalette = *p;
-        settings.refreshRate = profile.refreshRate;
-        settings.primaryScreen.allowReflowOnResize = config.reflowOnResize;
-        settings.highlightDoubleClickedWord = profile.highlightDoubleClickedWord;
-        settings.highlightTimeout = profile.highlightTimeout;
-        settings.frozenModes = profile.frozenModes;
+        settings.primaryScreen.allowReflowOnResize = config.reflowOnResize.value();
+        settings.highlightDoubleClickedWord = profile.highlightDoubleClickedWord.value();
+        settings.highlightTimeout = profile.highlightTimeout.value();
+        settings.frozenModes = profile.frozenModes.value();
 
         return settings;
     }
@@ -187,6 +192,8 @@ TerminalSession::TerminalSession(unique_ptr<vtpty::Pty> pty, ContourGuiApp& app)
     _profile { *_config.profile(_profileName) },
     _app { app },
     _currentColorPreference { app.colorPreference() },
+    _accumulatedScrollX { 0 },
+    _accumulatedScrollY { 0 },
     _terminal { *this,
                 std::move(pty),
                 createSettingsFromConfig(_config, _profile, _currentColorPreference),
@@ -195,10 +202,9 @@ TerminalSession::TerminalSession(unique_ptr<vtpty::Pty> pty, ContourGuiApp& app)
 {
     if (app.liveConfig())
     {
-        sessionLog()("Enable live configuration reloading of file {}.",
-                     _config.backingFilePath.generic_string());
+        sessionLog()("Enable live configuration reloading of file {}.", _config.configFile.generic_string());
         _configFileChangeWatcher = make_unique<QFileSystemWatcher>();
-        _configFileChangeWatcher->addPath(QString::fromStdString(_config.backingFilePath.generic_string()));
+        _configFileChangeWatcher->addPath(QString::fromStdString(_config.configFile.generic_string()));
         connect(_configFileChangeWatcher.get(),
                 SIGNAL(fileChanged(const QString&)),
                 this,
@@ -287,7 +293,6 @@ void TerminalSession::mainLoop()
     }
 
     sessionLog()("Event loop terminating (PTY {}).", _terminal.device().isClosed() ? "closed" : "open");
-    onClosed();
 }
 
 void TerminalSession::terminate()
@@ -302,9 +307,9 @@ void TerminalSession::terminate()
 // {{{ Events implementations
 void TerminalSession::bell()
 {
-    emit onBell(_profile.bell.volume);
+    emit onBell(_profile.bell.value().volume);
 
-    if (_profile.bell.alert)
+    if (_profile.bell.value().alert)
         emit onAlert();
 }
 
@@ -323,7 +328,7 @@ void TerminalSession::screenUpdated()
     if (!_display)
         return;
 
-    if (_profile.autoScrollOnUpdate && terminal().viewport().scrolled()
+    if (_profile.autoScrollOnUpdate.value() && terminal().viewport().scrolled()
         && terminal().inputHandler().mode() == ViMode::Insert)
         terminal().viewport().scrollToBottom();
 
@@ -410,7 +415,7 @@ void TerminalSession::updateColorPreference(vtbackend::ColorPreference preferenc
         return;
 
     _currentColorPreference = preference;
-    if (auto const* colorPalette = preferredColorPalette(_profile.colors, preference))
+    if (auto const* colorPalette = preferredColorPalette(_profile.colors.value(), preference))
     {
         _terminal.resetColorPalette(*colorPalette);
 
@@ -454,7 +459,7 @@ void TerminalSession::requestShowHostWritableStatusLine()
 {
     if (_display)
         _display->post([this]() {
-            requestPermission(_profile.permissions.displayHostWritableStatusLine,
+            requestPermission(_profile.displayHostWritableStatusLine.value(),
                               GuardedRole::ShowHostWritableStatusLine);
         });
 }
@@ -470,7 +475,7 @@ void TerminalSession::executeShowHostWritableStatusLine(bool allow, bool remembe
     _terminal.setStatusDisplay(vtbackend::StatusDisplayType::HostWritable);
     displayLog()("requestCaptureBuffer: Finished. Waking up I/O thread.");
     flushInput();
-    _terminal.state().syncWindowTitleWithHostWritableStatusDisplay = false;
+    _terminal.setSyncWindowTitleWithHostWritableStatusDisplay(false);
 }
 
 vtbackend::FontDef TerminalSession::getFontDef()
@@ -485,7 +490,7 @@ void TerminalSession::setFontDef(vtbackend::FontDef const& fontDef)
 
     _pendingFontChange = fontDef;
 
-    _display->post([this]() { requestPermission(_profile.permissions.changeFont, GuardedRole::ChangeFont); });
+    _display->post([this]() { requestPermission(_profile.changeFont.value(), GuardedRole::ChangeFont); });
 }
 
 void TerminalSession::applyPendingFontChange(bool allow, bool remember)
@@ -496,7 +501,7 @@ void TerminalSession::applyPendingFontChange(bool allow, bool remember)
     if (!_pendingFontChange)
         return;
 
-    auto const& currentFonts = _profile.fonts;
+    auto const& currentFonts = _profile.fonts.value();
     vtrasterizer::FontDescriptions newFonts = currentFonts;
 
     auto const spec = std::move(_pendingFontChange.value());
@@ -544,7 +549,6 @@ void TerminalSession::copyToClipboard(std::string_view data)
 
 void TerminalSession::openDocument(std::string_view fileOrUrl)
 {
-    auto stripped = strip_if(string(fileOrUrl), true);
     sessionLog()("openDocument: {}\n", fileOrUrl);
     QDesktopServices::openUrl(QUrl(QString::fromStdString(std::string(fileOrUrl))));
 }
@@ -571,15 +575,6 @@ void TerminalSession::notify(string_view title, string_view content)
 void TerminalSession::onClosed()
 {
     auto const _ = std::scoped_lock { _onClosedMutex };
-    auto isClosedAlready = _onClosedHandled.load();
-    if (isClosedAlready || !_onClosedHandled.compare_exchange_weak(isClosedAlready, true))
-    {
-        sessionLog()("onClosed called: thread {}, display {}", crispy::threadName(), _display ? "yes" : "no");
-        if (_display)
-            _display->closeDisplay();
-        return;
-    }
-
     sessionLog()("Terminal device closed (thread {})", crispy::threadName());
 
     if (!_terminal.device().isClosed())
@@ -621,12 +616,21 @@ void TerminalSession::onClosed()
         auto constexpr TextLines = array<string_view, 2> { "Shell terminated too quickly.",
                                                            "The window will not be closed automatically." };
         for (auto const text: TextLines)
-            _terminal.writeToScreen(fmt::format("\r\n{}{}{}", SGR, EL, text));
+            _terminal.writeToScreen(std::format("\r\n{}{}{}", SGR, EL, text));
         _terminal.writeToScreen("\r\n");
         _terminatedAndWaitingForKeyPress = true;
         return;
     }
 
+    auto isClosedAlready = _onClosedHandled.load();
+    if (isClosedAlready || !_onClosedHandled.compare_exchange_weak(isClosedAlready, true))
+    {
+        sessionLog()("onClosed called: thread {}, display {}", crispy::threadName(), _display ? "yes" : "no");
+        if (_display)
+            _display->closeDisplay();
+        return;
+    }
+
     if (_app.dumpStateAtExit().has_value())
         inspect();
     else if (_display)
@@ -665,7 +669,7 @@ void TerminalSession::pasteFromClipboard(unsigned count, bool strip)
 
 void TerminalSession::onSelectionCompleted()
 {
-    switch (_config.onMouseSelection)
+    switch (_config.onMouseSelection.value())
     {
         case config::SelectionAction::CopyToSelectionClipboard:
             if (QClipboard* clipboard = QGuiApplication::clipboard();
@@ -747,11 +751,11 @@ void TerminalSession::inputModeChanged(vtbackend::ViMode mode)
     using vtbackend::ViMode;
     switch (mode)
     {
-        case ViMode::Insert: configureCursor(_profile.inputModes.insert.cursor); break;
-        case ViMode::Normal: configureCursor(_profile.inputModes.normal.cursor); break;
+        case ViMode::Insert: configureCursor(_profile.modeInsert.value().cursor); break;
+        case ViMode::Normal: configureCursor(_profile.modeNormal.value().cursor); break;
         case ViMode::Visual:
         case ViMode::VisualLine:
-        case ViMode::VisualBlock: configureCursor(_profile.inputModes.visual.cursor); break;
+        case ViMode::VisualBlock: configureCursor(_profile.modeVisual.value().cursor); break;
     }
 }
 
@@ -772,13 +776,13 @@ void TerminalSession::sendKeyEvent(Key key, Modifiers modifiers, KeyboardEventTy
         return;
     }
 
-    if (_profile.mouseHideWhileTyping)
+    if (_profile.mouseHideWhileTyping.value())
         _display->setMouseCursorShape(MouseCursorShape::Hidden);
 
     if (eventType != KeyboardEventType::Release)
     {
         if (auto const* actions =
-                config::apply(_config.inputMappings.keyMappings, key, modifiers, matchModeFlags()))
+                config::apply(_config.inputMappings.value().keyMappings, key, modifiers, matchModeFlags()))
         {
             executeAllActions(*actions);
             return;
@@ -804,13 +808,15 @@ void TerminalSession::sendCharEvent(
         return;
     }
 
-    if (_profile.mouseHideWhileTyping)
+    if (_profile.mouseHideWhileTyping.value())
         _display->setMouseCursorShape(MouseCursorShape::Hidden);
 
     if (eventType != KeyboardEventType::Release)
     {
+        // find if action exist for the given key, and ignore if editing search prompt
         if (auto const* actions =
-                config::apply(_config.inputMappings.charMappings, value, modifiers, matchModeFlags()))
+                config::apply(_config.inputMappings.value().charMappings, value, modifiers, matchModeFlags());
+            actions && !_terminal.inputHandler().isEditingSearch())
         {
             executeAllActions(*actions);
             return;
@@ -833,12 +839,12 @@ void TerminalSession::sendMousePressEvent(Modifiers modifiers,
         }))
         return;
 
-    auto const sanitizedModifier = modifiers.contains(_config.bypassMouseProtocolModifiers)
-                                       ? modifiers.without(_config.bypassMouseProtocolModifiers)
+    auto const sanitizedModifier = modifiers.contains(_config.bypassMouseProtocolModifiers.value())
+                                       ? modifiers.without(_config.bypassMouseProtocolModifiers.value())
                                        : modifiers;
 
-    if (auto const* actions =
-            config::apply(_config.inputMappings.mouseMappings, button, sanitizedModifier, matchModeFlags()))
+    if (auto const* actions = config::apply(
+            _config.inputMappings.value().mouseMappings, button, sanitizedModifier, matchModeFlags()))
         executeAllActions(*actions);
 }
 
@@ -891,7 +897,7 @@ void TerminalSession::sendFocusInEvent()
     terminal().sendFocusInEvent();
 
     if (_display)
-        _display->setBlurBehind(_profile.backgroundBlur);
+        _display->setBlurBehind(_profile.backgroundBlur.value());
 
     scheduleRedraw();
 }
@@ -985,10 +991,16 @@ bool TerminalSession::operator()(actions::CreateDebugDump)
     return true;
 }
 
+bool TerminalSession::operator()(actions::CreateSelection const& customSelector)
+{
+    _terminal.triggerWordWiseSelectionWithCustomDelimiters(customSelector.delimiters);
+    return true;
+}
+
 bool TerminalSession::operator()(actions::DecreaseFontSize)
 {
     auto constexpr OnePt = text::font_size { 1.0 };
-    setFontSize(profile().fonts.size - OnePt);
+    setFontSize(profile().fonts.value().size - OnePt);
 
     emit fontSizeChanged();
     // auto const currentFontSize = view().renderer().fontDescriptions().size;
@@ -999,10 +1011,10 @@ bool TerminalSession::operator()(actions::DecreaseFontSize)
 
 bool TerminalSession::operator()(actions::DecreaseOpacity)
 {
-    if (static_cast<uint8_t>(_profile.backgroundOpacity) == 0)
+    if (static_cast<uint8_t>(_profile.backgroundOpacity.value()) == 0)
         return true;
 
-    --_profile.backgroundOpacity;
+    --_profile.backgroundOpacity.value();
 
     emit opacityChanged();
 
@@ -1015,16 +1027,22 @@ bool TerminalSession::operator()(actions::DecreaseOpacity)
 
 bool TerminalSession::operator()(actions::FocusNextSearchMatch)
 {
-    if (_terminal.state().viCommands.jumpToNextMatch(1))
-        _terminal.viewport().makeVisibleWithinSafeArea(_terminal.state().viCommands.cursorPosition.line);
+    auto const nextPosition = _terminal.searchNextMatch(_terminal.normalModeCursorPosition());
+    if (!nextPosition)
+        return false;
+    _terminal.moveNormalModeCursorTo(nextPosition.value());
+    _terminal.viewport().makeVisibleWithinSafeArea(nextPosition->line);
     // TODO why didn't the makeVisibleWithinSafeArea() call from inside jumpToNextMatch not work?
     return true;
 }
 
 bool TerminalSession::operator()(actions::FocusPreviousSearchMatch)
 {
-    if (_terminal.state().viCommands.jumpToPreviousMatch(1))
-        _terminal.viewport().makeVisibleWithinSafeArea(_terminal.state().viCommands.cursorPosition.line);
+    auto const nextPosition = _terminal.searchPrevMatch(_terminal.normalModeCursorPosition());
+    if (!nextPosition)
+        return false;
+    _terminal.moveNormalModeCursorTo(nextPosition.value());
+    _terminal.viewport().makeVisibleWithinSafeArea(nextPosition->line);
     // TODO why didn't the makeVisibleWithinSafeArea() call from inside jumpToPreviousMatch not work?
     return true;
 }
@@ -1048,15 +1066,15 @@ bool TerminalSession::operator()(actions::IncreaseFontSize)
     // setFontSize(newFontSize);
 
     emit fontSizeChanged();
-    setFontSize(profile().fonts.size + OnePt);
+    setFontSize(profile().fonts.value().size + OnePt);
     return true;
 }
 
 bool TerminalSession::operator()(actions::IncreaseOpacity)
 {
-    if (static_cast<uint8_t>(_profile.backgroundOpacity) >= std::numeric_limits<uint8_t>::max())
+    if (static_cast<uint8_t>(_profile.backgroundOpacity.value()) >= std::numeric_limits<uint8_t>::max())
         return true;
-    ++_profile.backgroundOpacity;
+    ++_profile.backgroundOpacity.value();
 
     emit opacityChanged();
 
@@ -1079,10 +1097,10 @@ bool TerminalSession::operator()(actions::NoSearchHighlight)
     return true;
 }
 
-bool TerminalSession::operator()(actions::OpenConfiguration)
+bool TerminalSession::operator()(actions::OpenConfiguration) const
 {
-    if (!QDesktopServices::openUrl(QUrl(QString::fromUtf8(_config.backingFilePath.string().c_str()))))
-        errorLog()("Could not open configuration file \"{}\".", _config.backingFilePath.generic_string());
+    if (!QDesktopServices::openUrl(QUrl(QString::fromUtf8(_config.configFile.string().c_str()))))
+        errorLog()("Could not open configuration file \"{}\".", _config.configFile.generic_string());
 
     return true;
 }
@@ -1111,12 +1129,15 @@ bool TerminalSession::operator()(actions::PasteClipboard paste)
     return true;
 }
 
-bool TerminalSession::operator()(actions::PasteSelection)
+bool TerminalSession::operator()(actions::PasteSelection paste)
 {
     if (QClipboard* clipboard = QGuiApplication::clipboard(); clipboard != nullptr)
     {
         string const text = normalize_crlf(clipboard->text(QClipboard::Selection));
-        terminal().sendPaste(string_view { text });
+        if (paste.evaluateInShell)
+            terminal().sendRawInput(string_view { text + "\n" });
+        else
+            terminal().sendPaste(string_view { text });
     }
 
     return true;
@@ -1148,7 +1169,7 @@ bool TerminalSession::operator()(actions::ResetConfig)
 bool TerminalSession::operator()(actions::ResetFontSize)
 {
     if (config::TerminalProfile const* profile = _config.profile(_profileName))
-        setFontSize(profile->fonts.size);
+        setFontSize(profile->fonts.value().size);
     return true;
 }
 
@@ -1164,7 +1185,7 @@ bool TerminalSession::operator()(actions::ScreenshotVT)
 
 bool TerminalSession::operator()(actions::ScrollDown)
 {
-    terminal().viewport().scrollDown(_profile.historyScrollMultiplier);
+    terminal().viewport().scrollDown(_profile.historyScrollMultiplier.value());
     return true;
 }
 
@@ -1220,13 +1241,13 @@ bool TerminalSession::operator()(actions::ScrollToTop)
 
 bool TerminalSession::operator()(actions::ScrollUp)
 {
-    terminal().viewport().scrollUp(_profile.historyScrollMultiplier);
+    terminal().viewport().scrollUp(_profile.historyScrollMultiplier.value());
     return true;
 }
 
 bool TerminalSession::operator()(actions::SearchReverse)
 {
-    _terminal.inputHandler().startSearchExternally();
+    terminal().inputHandler().startSearchExternally();
 
     return true;
 }
@@ -1244,6 +1265,15 @@ bool TerminalSession::operator()(actions::ToggleAllKeyMaps)
 {
     _allowKeyMappings = !_allowKeyMappings;
     inputLog()("{} key mappings.", _allowKeyMappings ? "Enabling" : "Disabling");
+
+    if (!_allowKeyMappings)
+    {
+        terminal().setStatusLineDefinition(
+            parseStatusLineDefinition("{}", "{Text:text=key bindings disabled,Left= « ,Right= » }", "{}"));
+        terminal().setStatusDisplay(StatusDisplayType::Indicator);
+    }
+    else
+        terminal().resetStatusLineDefinition();
     return true;
 }
 
@@ -1263,7 +1293,7 @@ bool TerminalSession::operator()(actions::ToggleInputProtection)
 bool TerminalSession::operator()(actions::ToggleStatusLine)
 {
     auto const l = scoped_lock { _terminal };
-    if (terminal().state().statusDisplayType != StatusDisplayType::Indicator)
+    if (terminal().statusDisplayType() != StatusDisplayType::Indicator)
         terminal().setStatusDisplay(StatusDisplayType::Indicator);
     else
         terminal().setStatusDisplay(StatusDisplayType::None);
@@ -1271,8 +1301,8 @@ bool TerminalSession::operator()(actions::ToggleStatusLine)
     // `savedStatusDisplayType` holds only a value if the application has been overriding
     // the status display type. But the user now actively requests a given type,
     // so make sure restoring will not destroy the user's desire.
-    if (terminal().state().savedStatusDisplayType)
-        terminal().state().savedStatusDisplayType = terminal().state().statusDisplayType;
+    if (terminal().savedStatusDisplayType())
+        terminal().setSavedStatusDisplayType(terminal().statusDisplayType());
 
     return true;
 }
@@ -1332,7 +1362,7 @@ void TerminalSession::setDefaultCursor()
         return;
 
     using Type = vtbackend::ScreenType;
-    switch (terminal().screenType())
+    switch (_terminal.screenType())
     {
         case Type::Primary: _display->setMouseCursorShape(MouseCursorShape::IBeam); break;
         case Type::Alternate: _display->setMouseCursorShape(MouseCursorShape::Arrow); break;
@@ -1343,7 +1373,7 @@ bool TerminalSession::reloadConfig(config::Config newConfig, string const& profi
 {
     // clang-format off
     sessionLog()("Reloading configuration from {} with profile {}",
-                 newConfig.backingFilePath.string(), profileName);
+                 newConfig.configFile.string(), profileName);
     // clang-format on
 
     _config = std::move(newConfig);
@@ -1407,16 +1437,15 @@ void TerminalSession::spawnNewTerminal(string const& profileName)
         return "."s;
     }();
 
-    if (_config.spawnNewProcess)
+    if (_config.spawnNewProcess.value())
     {
         sessionLog()("spawning new process");
-        ::contour::spawnNewTerminal(
-            _app.programPath(), _config.backingFilePath.generic_string(), profileName, wd);
+        ::contour::spawnNewTerminal(_app.programPath(), _config.configFile.generic_string(), profileName, wd);
     }
     else
     {
         sessionLog()("spawning new in-process window");
-        _app.config().profile(_profileName)->shell.workingDirectory = fs::path(wd);
+        _app.config().profile(_profileName)->shell.value().workingDirectory = fs::path(wd);
         _app.newWindow();
     }
 }
@@ -1442,28 +1471,31 @@ void TerminalSession::configureTerminal()
     auto const l = scoped_lock { _terminal };
     sessionLog()("Configuring terminal.");
 
-    _terminal.setWordDelimiters(_config.wordDelimiters);
-    _terminal.setMouseProtocolBypassModifiers(_config.bypassMouseProtocolModifiers);
-    _terminal.setMouseBlockSelectionModifiers(_config.mouseBlockSelectionModifiers);
-    _terminal.setLastMarkRangeOffset(_profile.copyLastMarkRangeOffset);
+    _terminal.setWordDelimiters(_config.wordDelimiters.value());
+    _terminal.setExtendedWordDelimiters(_config.extendedWordDelimiters.value());
+    _terminal.setMouseProtocolBypassModifiers(_config.bypassMouseProtocolModifiers.value());
+    _terminal.setMouseBlockSelectionModifiers(_config.mouseBlockSelectionModifiers.value());
+    _terminal.setLastMarkRangeOffset(_profile.copyLastMarkRangeOffset.value());
 
-    sessionLog()("Setting terminal ID to {}.", _profile.terminalId);
-    _terminal.setTerminalId(_profile.terminalId);
-    _terminal.setMaxImageColorRegisters(_config.maxImageColorRegisters);
-    _terminal.setMaxImageSize(_config.maxImageSize);
-    _terminal.setMode(vtbackend::DECMode::NoSixelScrolling, !_config.sixelScrolling);
-    _terminal.setStatusDisplay(_profile.initialStatusDisplayType);
-    sessionLog()("maxImageSize={}, sixelScrolling={}", _config.maxImageSize, _config.sixelScrolling);
+    sessionLog()("Setting terminal ID to {}.", _profile.terminalId.value());
+    _terminal.setTerminalId(_profile.terminalId.value());
+    _terminal.setMaxSixelColorRegisters(_config.maxImageColorRegisters.value());
+    _terminal.setMaxImageSize(_config.maxImageSize.value());
+    _terminal.setMode(vtbackend::DECMode::NoSixelScrolling, !_config.sixelScrolling.value());
+    _terminal.setStatusDisplay(_profile.initialStatusDisplayType.value());
+    sessionLog()(
+        "maxImageSize={}, sixelScrolling={}", _config.maxImageSize.value(), _config.sixelScrolling.value());
 
     // XXX
     // if (!terminalView.renderer().renderTargetAvailable())
     //     return;
 
-    configureCursor(_profile.inputModes.insert.cursor);
+    configureCursor(_profile.modeInsert.value().cursor);
     updateColorPreference(_app.colorPreference());
-    _terminal.setMaxHistoryLineCount(_profile.maxHistoryLineCount);
-    _terminal.setHighlightTimeout(_profile.highlightTimeout);
-    _terminal.viewport().setScrollOff(_profile.modalCursorScrollOff);
+    _terminal.setMaxHistoryLineCount(_profile.maxHistoryLineCount.value());
+    _terminal.setHighlightTimeout(_profile.highlightTimeout.value());
+    _terminal.viewport().setScrollOff(_profile.modalCursorScrollOff.value());
+    _terminal.inputHandler().setSearchModeSwitch(_profile.searchModeSwitch.value());
 }
 
 void TerminalSession::configureCursor(config::CursorConfig const& cursorConfig)
@@ -1483,7 +1515,7 @@ void TerminalSession::configureDisplay()
         return;
 
     sessionLog()("Configuring display.");
-    _display->setBlurBehind(_profile.backgroundBlur);
+    _display->setBlurBehind(_profile.backgroundBlur.value());
 
     {
         auto const dpr = _display->contentScale();
@@ -1493,19 +1525,20 @@ void TerminalSession::configureDisplay()
         _terminal.setMaxImageSize(actualScreenSize, actualScreenSize);
     }
 
-    if (_profile.maximized)
+    if (_profile.maximized.value())
         _display->setWindowMaximized();
     else
         _display->setWindowNormal();
 
-    if (_profile.fullscreen != _display->isFullScreen())
+    if (_profile.fullscreen.value() != _display->isFullScreen())
         _display->toggleFullScreen();
 
     _terminal.setRefreshRate(_display->refreshRate());
-    _display->setFonts(_profile.fonts);
+    _display->setFonts(_profile.fonts.value());
     adaptToWidgetSize();
 
-    _display->setHyperlinkDecoration(_profile.hyperlinkDecoration.normal, _profile.hyperlinkDecoration.hover);
+    _display->setHyperlinkDecoration(_profile.hyperlinkDecorationNormal.value(),
+                                     _profile.hyperlinkDecorationHover.value());
 
     setWindowTitle(_terminal.windowTitle());
 }
@@ -1529,7 +1562,7 @@ uint8_t TerminalSession::matchModeFlags() const
     if (_terminal.inputHandler().mode() == ViMode::Insert)
         flags |= static_cast<uint8_t>(MatchModes::Flag::Insert);
 
-    if (!_terminal.state().searchMode.pattern.empty())
+    if (!_terminal.search().pattern.empty())
         flags |= static_cast<uint8_t>(MatchModes::Flag::Search);
 
     if (_terminal.executionMode() != ExecutionMode::Normal)
@@ -1543,7 +1576,7 @@ void TerminalSession::setFontSize(text::font_size size)
     if (!_display->setFontSize(size))
         return;
 
-    _profile.fonts.size = size;
+    _profile.fonts.value().size = size;
 }
 
 bool TerminalSession::reloadConfigWithProfile(string const& profileName)
@@ -1553,7 +1586,7 @@ bool TerminalSession::reloadConfigWithProfile(string const& profileName)
 
     try
     {
-        loadConfigFromFile(newConfig, _config.backingFilePath.string());
+        loadConfigFromFile(newConfig, _config.configFile.string());
     }
     catch (exception const& e)
     {
@@ -1564,7 +1597,7 @@ bool TerminalSession::reloadConfigWithProfile(string const& profileName)
 
     if (!newConfig.profile(profileName))
     {
-        errorLog()(fmt::format("Currently active profile with name '{}' gone.", profileName));
+        errorLog()(std::format("Currently active profile with name '{}' gone.", profileName));
         ++configFailures;
     }
 
@@ -1579,11 +1612,11 @@ bool TerminalSession::reloadConfigWithProfile(string const& profileName)
 
 bool TerminalSession::resetConfig()
 {
-    auto const ec = config::createDefaultConfig(_config.backingFilePath);
+    auto const ec = config::createDefaultConfig(_config.configFile);
     if (ec)
     {
         errorLog()("Failed to load default config at {}; ({}) {}",
-                   _config.backingFilePath.string(),
+                   _config.configFile.string(),
                    ec.category().name(),
                    ec.message());
         return false;
@@ -1592,14 +1625,14 @@ bool TerminalSession::resetConfig()
     config::Config const defaultConfig;
     try
     {
-        config::loadConfigFromFile(_config.backingFilePath);
+        config::loadConfigFromFile(_config.configFile);
     }
     catch (exception const& e)
     {
         sessionLog()("Failed to load default config: {}", e.what());
     }
 
-    return reloadConfig(defaultConfig, defaultConfig.defaultProfileName);
+    return reloadConfig(defaultConfig, defaultConfig.defaultProfileName.value());
 }
 
 void TerminalSession::followHyperlink(vtbackend::HyperlinkInfo const& hyperlink)
@@ -1612,7 +1645,7 @@ void TerminalSession::followHyperlink(vtbackend::HyperlinkInfo const& hyperlink)
     {
         QStringList args;
         args.append("config");
-        args.append(QString::fromStdString(_config.backingFilePath.string()));
+        args.append(QString::fromStdString(_config.configFile.string()));
         args.append(QString::fromUtf8(hyperlink.path().data(), static_cast<int>(hyperlink.path().size())));
         QProcess::execute(QString::fromStdString(_app.programPath()), args);
     }
@@ -1620,7 +1653,7 @@ void TerminalSession::followHyperlink(vtbackend::HyperlinkInfo const& hyperlink)
     {
         QStringList args;
         args.append("config");
-        args.append(QString::fromStdString(_config.backingFilePath.string()));
+        args.append(QString::fromStdString(_config.configFile.string()));
         args.append(QString::fromStdString(editorEnv));
         args.append(QString::fromUtf8(hyperlink.path().data(), static_cast<int>(hyperlink.path().size())));
         QProcess::execute(QString::fromStdString(_app.programPath()), args);
