@@ -12,14 +12,13 @@
 
 #include <crispy/point.h>
 
-#include <fmt/format.h>
-
 #include <QtCore/QAbstractItemModel>
 #include <QtCore/QFileSystemWatcher>
 #include <QtCore/QThread>
 #include <QtQml/QJSValue>
 
 #include <cstdint>
+#include <format>
 #include <thread>
 
 #include <qcolor.h>
@@ -37,7 +36,7 @@ class ContourGuiApp;
 /**
  * A set of user-facing activities that are guarded behind a permission-check wall.
  */
-enum class GuardedRole
+enum class GuardedRole : uint8_t
 {
     ChangeFont,
     CaptureBuffer,
@@ -78,6 +77,7 @@ class TerminalSession: public QAbstractItemModel, public vtbackend::Terminal::Ev
     Q_PROPERTY(bool isScrollbarVisible READ getIsScrollbarVisible NOTIFY isScrollbarVisibleChanged)
     Q_PROPERTY(int fontSize READ getFontSize)
     Q_PROPERTY(int upTime READ getUptime)
+    Q_PROPERTY(QString bellSource READ getBellSource NOTIFY onBell)
 
     // Q_PROPERTY(QString profileName READ profileName NOTIFY profileNameChanged)
 
@@ -91,10 +91,24 @@ class TerminalSession: public QAbstractItemModel, public vtbackend::Terminal::Ev
         return static_cast<int>(diff.count());
     }
 
-    int getFontSize() const noexcept { return static_cast<int>(_profile.fonts.size.pt); }
+    QString getBellSource() const noexcept
+    {
+        if (_profile.bell.value().sound == "default")
+        {
+            return QString("qrc:/contour/bell.wav");
+        }
+        if (_profile.bell.value().sound == "off")
+        {
+            return QString();
+        }
+
+        return QString::fromStdString(_profile.bell.value().sound);
+    }
+
+    int getFontSize() const noexcept { return static_cast<int>(_profile.fonts.value().size.pt); }
     float getOpacity() const noexcept
     {
-        return static_cast<float>(_profile.backgroundOpacity) / std::numeric_limits<uint8_t>::max();
+        return static_cast<float>(_profile.backgroundOpacity.value()) / std::numeric_limits<uint8_t>::max();
     }
     QString pathToBackground() const
     {
@@ -111,39 +125,47 @@ class TerminalSession: public QAbstractItemModel, public vtbackend::Terminal::Ev
         auto color = terminal().isModeEnabled(vtbackend::DECMode::ReverseVideo)
                          ? _terminal.colorPalette().defaultForeground
                          : _terminal.colorPalette().defaultBackground;
-        return QColor(color.red, color.green, color.blue, static_cast<uint8_t>(_profile.backgroundOpacity));
+        auto alpha = static_cast<uint8_t>(_profile.backgroundOpacity.value());
+        return QColor(color.red, color.green, color.blue, alpha);
     }
     float getOpacityBackground() const noexcept
     {
-        if (_terminal.colorPalette().backgroundImage)
+        if (_terminal.colorPalette().backgroundImage.get())
+        {
             return _terminal.colorPalette().backgroundImage->opacity;
+        }
         return 0.0;
     }
     bool getIsImageBackground() const noexcept
     {
         if (_terminal.colorPalette().backgroundImage)
+        {
             return true;
+        }
         return false;
     }
 
     bool getIsBlurBackground() const noexcept
     {
         if (getIsImageBackground())
+        {
             return _terminal.colorPalette().backgroundImage->blur;
+        }
         return false;
     }
 
     bool getIsScrollbarRight() const noexcept
     {
-        return profile().scrollbarPosition == config::ScrollBarPosition::Right;
+        return profile().scrollbarPosition.value() == config::ScrollBarPosition::Right;
     }
 
     bool getIsScrollbarVisible() const noexcept
     {
-        if (profile().scrollbarPosition == config::ScrollBarPosition::Hidden)
+        if (profile().scrollbarPosition.value() == config::ScrollBarPosition::Hidden)
             return false;
 
-        if ((_currentScreenType == vtbackend::ScreenType::Alternate) && profile().hideScrollbarInAltScreen)
+        if ((_currentScreenType == vtbackend::ScreenType::Alternate)
+            && profile().hideScrollbarInAltScreen.value())
             return false;
 
         return true;
@@ -164,7 +186,7 @@ class TerminalSession: public QAbstractItemModel, public vtbackend::Terminal::Ev
 
     int pageColumnsCount() const noexcept { return unbox(_terminal.pageSize().columns); }
 
-    bool showResizeIndicator() const noexcept { return _config.profile().sizeIndicatorOnResize; }
+    bool showResizeIndicator() const noexcept { return _config.profile().sizeIndicatorOnResize.value(); }
 
     int historyLineCount() const noexcept { return unbox(_terminal.currentScreen().historyLineCount()); }
 
@@ -220,26 +242,26 @@ class TerminalSession: public QAbstractItemModel, public vtbackend::Terminal::Ev
     void attachDisplay(display::TerminalDisplay& display);
     void detachDisplay(display::TerminalDisplay& display);
 
-    Q_INVOKABLE void applyPendingFontChange(bool answer, bool remember);
-    Q_INVOKABLE void executePendingBufferCapture(bool answer, bool remember);
-    Q_INVOKABLE void executeShowHostWritableStatusLine(bool answer, bool remember);
+    Q_INVOKABLE void applyPendingFontChange(bool allow, bool remember);
+    Q_INVOKABLE void executePendingBufferCapture(bool allow, bool remember);
+    Q_INVOKABLE void executeShowHostWritableStatusLine(bool allow, bool remember);
     Q_INVOKABLE void adaptToWidgetSize();
 
     void updateColorPreference(vtbackend::ColorPreference preference);
 
     // vtbackend::Events
     //
-    void requestCaptureBuffer(vtbackend::LineCount lineCount, bool logical) override;
+    void requestCaptureBuffer(vtbackend::LineCount lines, bool logical) override;
     void bell() override;
     void bufferChanged(vtbackend::ScreenType) override;
     void renderBufferUpdated() override;
     void screenUpdated() override;
     vtbackend::FontDef getFontDef() override;
-    void setFontDef(vtbackend::FontDef const& fontSpec) override;
+    void setFontDef(vtbackend::FontDef const& fontDef) override;
     void copyToClipboard(std::string_view data) override;
     void openDocument(std::string_view /*fileOrUrl*/) override;
     void inspect() override;
-    void notify(std::string_view title, std::string_view body) override;
+    void notify(std::string_view title, std::string_view content) override;
     void onClosed() override;
     void pasteFromClipboard(unsigned count, bool strip) override;
     void onSelectionCompleted() override;
@@ -288,6 +310,7 @@ class TerminalSession: public QAbstractItemModel, public vtbackend::Terminal::Ev
     bool operator()(actions::CopyPreviousMarkRange);
     bool operator()(actions::CopySelection);
     bool operator()(actions::CreateDebugDump);
+    bool operator()(actions::CreateSelection const&);
     bool operator()(actions::DecreaseFontSize);
     bool operator()(actions::DecreaseOpacity);
     bool operator()(actions::FollowHyperlink);
@@ -297,7 +320,7 @@ class TerminalSession: public QAbstractItemModel, public vtbackend::Terminal::Ev
     bool operator()(actions::IncreaseOpacity);
     bool operator()(actions::NewTerminal const&);
     bool operator()(actions::NoSearchHighlight);
-    bool operator()(actions::OpenConfiguration);
+    bool operator()(actions::OpenConfiguration) const;
     bool operator()(actions::OpenFileManager);
     bool operator()(actions::OpenSelection);
     bool operator()(actions::PasteClipboard);
@@ -448,31 +471,21 @@ class TerminalSession: public QAbstractItemModel, public vtbackend::Terminal::Ev
 
 Q_DECLARE_INTERFACE(contour::TerminalSession, "org.contour.TerminalSession")
 
-namespace fmt
-{
-
 template <>
-struct formatter<contour::GuardedRole>
+struct std::formatter<contour::GuardedRole>: std::formatter<std::string_view>
 {
-    template <typename ParseContext>
-    constexpr auto parse(ParseContext& ctx)
-    {
-        return ctx.begin();
-    }
-
     template <typename FormatContext>
-    auto format(contour::GuardedRole value, FormatContext& ctx)
+    auto format(contour::GuardedRole value, FormatContext& ctx) const
     {
+        std::string_view output;
+        // clang-format off
         switch (value)
         {
-                // clang-format off
-            case contour::GuardedRole::ChangeFont: return fmt::format_to(ctx.out(), "Change Font");
-            case contour::GuardedRole::CaptureBuffer: return fmt::format_to(ctx.out(), "Capture Buffer");
-            case contour::GuardedRole::ShowHostWritableStatusLine:  return fmt::format_to(ctx.out(), "show Host Writable Statusline");
-                // clang-format on
+            case contour::GuardedRole::ChangeFont: output = "Change Font"; break;
+            case contour::GuardedRole::CaptureBuffer: output = "Capture Buffer"; break;
+            case contour::GuardedRole::ShowHostWritableStatusLine:  output = "show Host Writable Statusline"; break;
         }
-        crispy::unreachable();
+        // clang-format on
+        return formatter<string_view>::format(output, ctx);
     }
 };
-
-} // namespace fmt
