@@ -15,9 +15,6 @@
 #include <crispy/logstore.h>
 #include <crispy/utils.h>
 
-#include <fmt/chrono.h>
-#include <fmt/format.h>
-
 #include <QtCore/QDebug>
 #include <QtCore/QFileInfo>
 #include <QtCore/QFileSystemWatcher>
@@ -37,7 +34,9 @@
 #include <QtQml/QQmlContext>
 #include <QtQuick/QQuickWindow>
 
+#include <chrono>
 #include <filesystem>
+#include <format>
 #include <fstream>
 #include <string_view>
 #include <tuple>
@@ -45,7 +44,7 @@
 
 namespace fs = std::filesystem;
 
-// Temporarily disabled (I think it was OS/X that didn't like glDebugMessageCallback).
+// Temporarily disabled (I think it was macOS that didn't like glDebugMessageCallback).
 // #define CONTOUR_DEBUG_OPENGL 1
 
 #if defined(_MSC_VER)
@@ -116,7 +115,7 @@ namespace
     #if defined(GL_DEBUG_SOURCE_OTHER_ARB)
                 case GL_DEBUG_SOURCE_OTHER_ARB: return "other"s;
     #endif
-                default: return fmt::format("{}", _severity);
+                default: return std::format("{}", _severity);
             }
         }();
         string const typeName = [&]() {
@@ -140,7 +139,7 @@ namespace
     #if defined(GL_DEBUG_TYPE_OTHER)
                 case GL_DEBUG_TYPE_OTHER: return "other"s;
     #endif
-                default: return fmt::format("{}", _severity);
+                default: return std::format("{}", _severity);
             }
         }();
         string const debugSeverity = [&]() {
@@ -158,7 +157,7 @@ namespace
     #if defined(GL_DEBUG_SEVERITY_NOTIFICATION)
                 case GL_DEBUG_SEVERITY_NOTIFICATION: return "notification"s;
     #endif
-                default: return fmt::format("{}", _severity);
+                default: return std::format("{}", _severity);
             }
         }();
         auto const tag = []([[maybe_unused]] GLint _type) {
@@ -197,7 +196,7 @@ namespace
 
     std::string unhandledExceptionMessage(std::string_view const& where, exception const& e)
     {
-        return fmt::format("{}: Unhandled exception caught ({}). {}", where, typeid(e).name(), e.what());
+        return std::format("{}: Unhandled exception caught ({}). {}", where, typeid(e).name(), e.what());
     }
 
     void reportUnhandledException(std::string_view const& where, exception const& e)
@@ -273,14 +272,14 @@ void TerminalDisplay::setSession(TerminalSession* newSession)
                  "contentScale={}",
                  (void const*) this,
                  (void const*) newSession,
-                 newSession->profile().ssh.hostname.empty()
-                     ? fmt::format("program={}", newSession->profile().shell.program)
-                     : fmt::format("{}@{}:{}",
-                                   newSession->profile().ssh.username,
-                                   newSession->profile().ssh.hostname,
-                                   newSession->profile().ssh.port),
-                 newSession->profile().terminalSize,
-                 newSession->profile().fonts.size,
+                 newSession->profile().ssh.value().hostname.empty()
+                     ? std::format("program={}", newSession->profile().shell.value().program)
+                     : std::format("{}@{}:{}",
+                                   newSession->profile().ssh.value().username,
+                                   newSession->profile().ssh.value().hostname,
+                                   newSession->profile().ssh.value().port),
+                 newSession->profile().terminalSize.value(),
+                 newSession->profile().fonts.value().size,
                  contentScale());
 
     _session = newSession;
@@ -289,17 +288,17 @@ void TerminalDisplay::setSession(TerminalSession* newSession)
 
     _session->start();
 
-    window()->setFlag(Qt::FramelessWindowHint, !profile().showTitleBar);
+    window()->setFlag(Qt::FramelessWindowHint, !profile().showTitleBar.value());
 
     _renderer =
-        make_unique<vtrasterizer::Renderer>(newSession->profile().terminalSize,
-                                            sanitizeFontDescription(profile().fonts, fontDPI()),
+        make_unique<vtrasterizer::Renderer>(newSession->profile().terminalSize.value(),
+                                            sanitizeFontDescription(profile().fonts.value(), fontDPI()),
                                             _session->terminal().colorPalette(),
-                                            newSession->config().textureAtlasHashtableSlots,
-                                            newSession->config().textureAtlasTileCount,
-                                            newSession->config().textureAtlasDirectMapping,
-                                            newSession->profile().hyperlinkDecoration.normal,
-                                            newSession->profile().hyperlinkDecoration.hover
+                                            newSession->config().textureAtlasHashtableSlots.value(),
+                                            newSession->config().textureAtlasTileCount.value(),
+                                            newSession->config().textureAtlasDirectMapping.value(),
+                                            newSession->profile().hyperlinkDecorationNormal.value(),
+                                            newSession->profile().hyperlinkDecorationHover.value()
                                             // TODO: , WindowMargin(windowMargin_.left, windowMargin_.bottom);
         );
 
@@ -317,7 +316,7 @@ vtbackend::PageSize TerminalDisplay::windowSize() const noexcept
     if (!_session)
         return vtbackend::PageSize { LineCount(25), ColumnCount(80) };
 
-    return profile().terminalSize;
+    return profile().terminalSize.value();
 }
 
 void TerminalDisplay::sizeChanged()
@@ -460,7 +459,7 @@ void TerminalDisplay::logDisplayInfo()
 
     // clang-format off
     auto const fontSizeInPx = static_cast<int>(ceil((
-        profile().fonts.size.pt / 72.0) * average(fontDPI())
+        profile().fonts.value().size.pt / 72.0) * average(fontDPI())
     ));
     auto const normalScreenSize = vtbackend::ImageSize {
         Width::cast_from(window()->screen()->size().width()),
@@ -520,6 +519,17 @@ void TerminalDisplay::onBeforeSynchronize()
     if (!_session)
         return;
 
+    // find screen with biggest width
+    auto* screenToUse = window()->screen();
+    for (auto* screen: window()->screen()->virtualSiblings())
+    {
+        if (screen->size().width() > screenToUse->size().width())
+        {
+            screenToUse = screen;
+        }
+    }
+    window()->setScreen(screenToUse);
+
     if (!_renderTarget)
     {
         // This is the first call, so create the renderer (on demand) now.
@@ -535,7 +545,8 @@ void TerminalDisplay::onBeforeSynchronize()
     auto const windowSize = window()->size() * dpr;
     Require(width() > 1.0 && height() > 1.0);
 
-    auto const viewSize = ImageSize { Width::cast_from(width() * dpr), Height::cast_from(height() * dpr) };
+    auto const viewSize =
+        ImageSize { Width::cast_from(windowSize.width()), Height::cast_from(windowSize.height()) };
 
     _renderTarget->setRenderSize(
         ImageSize { Width::cast_from(windowSize.width()), Height::cast_from(windowSize.height()) });
@@ -579,13 +590,12 @@ void TerminalDisplay::createRenderer()
                      windowSize.height());
     }
 
-    _renderTarget = new OpenGLRenderer(
-        _session->profile().textShader.value_or(builtinShaderConfig(ShaderClass::Text)),
-        _session->profile().backgroundShader.value_or(builtinShaderConfig(ShaderClass::Background)),
-        precalculatedViewSize,
-        precalculatedTargetSize,
-        textureTileSize,
-        viewportMargin);
+    _renderTarget = new OpenGLRenderer(builtinShaderConfig(ShaderClass::Text),
+                                       builtinShaderConfig(ShaderClass::Background),
+                                       precalculatedViewSize,
+                                       precalculatedTargetSize,
+                                       textureTileSize,
+                                       viewportMargin);
     _renderTarget->setWindow(window());
     _renderer->setRenderTarget(*_renderTarget);
 
@@ -880,9 +890,9 @@ bool TerminalDisplay::event(QEvent* event)
     }
     catch (std::exception const& e)
     {
-        fmt::print("Unhandled exception for event {}: {}\n",
-                   (unsigned) event->type(),
-                   QMetaEnum::fromType<QEvent::Type>().valueToKey(event->type()));
+        std::cout << std::format("Unhandled exception for event {}: {}\n",
+                                 (unsigned) event->type(),
+                                 QMetaEnum::fromType<QEvent::Type>().valueToKey(event->type()));
         reportUnhandledException(__PRETTY_FUNCTION__, e);
         return false;
     }
@@ -946,13 +956,13 @@ double TerminalDisplay::contentScale() const
 ///
 /// @param terminalSize the terminal size in rows and columns
 /// @param cellSize     the size of a single cell in pixels (with content scale already applied)
-constexpr ImageSize computeRequiredSize(config::TerminalProfile::WindowMargins margins,
+constexpr ImageSize computeRequiredSize(config::WindowMargins margins,
                                         ImageSize cellSize,
                                         PageSize totalPageSize) noexcept
 {
     // We multiply by 2 because the margins are applied to both sides of the terminal.
-    auto const marginSize = ImageSize { vtbackend::Width::cast_from(margins.horizontal * 2),
-                                        vtbackend::Height::cast_from(margins.vertical * 2) };
+    auto const marginSize = ImageSize { vtbackend::Width::cast_from(unbox(margins.horizontal) * 2),
+                                        vtbackend::Height::cast_from(unbox(margins.vertical) * 2) };
 
     return (cellSize * totalPageSize + marginSize);
 }
@@ -963,7 +973,7 @@ void TerminalDisplay::updateImplicitSize()
     assert(_session);
     assert(window());
 
-    auto const requiredSize = computeRequiredSize(_session->profile().margins,
+    auto const requiredSize = computeRequiredSize(_session->profile().margins.value(),
                                                   _renderer->cellSize() * (1.0 / contentScale()),
                                                   _session->terminal().totalPageSize());
 
@@ -978,8 +988,9 @@ void TerminalDisplay::updateMinimumSize()
     assert(_session);
 
     auto constexpr MinimumTotalPageSize = PageSize { LineCount(5), ColumnCount(10) };
-    auto const minimumSize = computeRequiredSize(
-        _session->profile().margins, _renderer->cellSize() * (1.0 / contentScale()), MinimumTotalPageSize);
+    auto const minimumSize = computeRequiredSize(_session->profile().margins.value(),
+                                                 _renderer->cellSize() * (1.0 / contentScale()),
+                                                 MinimumTotalPageSize);
 
     window()->setMinimumSize(QSize(unbox<int>(minimumSize.width), unbox<int>(minimumSize.height)));
 }
@@ -990,14 +1001,10 @@ vtbackend::RefreshRate TerminalDisplay::refreshRate() const
 {
     auto* const screen = window()->screen();
     if (!screen)
-        return { profile().refreshRate.value != 0.0 ? profile().refreshRate
-                                                    : vtbackend::RefreshRate { 30.0 } };
+        return { vtbackend::RefreshRate { 30.0 } };
 
     auto const systemRefreshRate = vtbackend::RefreshRate { static_cast<double>(screen->refreshRate()) };
-    if (1.0 < profile().refreshRate.value && profile().refreshRate.value < systemRefreshRate.value)
-        return profile().refreshRate;
-    else
-        return systemRefreshRate;
+    return systemRefreshRate;
 }
 
 DPI TerminalDisplay::fontDPI() const noexcept
@@ -1077,7 +1084,7 @@ void TerminalDisplay::doDumpStateInternal()
 
     // clang-format off
     auto const targetBaseDir = _session->app().dumpStateAtExit().value_or(crispy::app::instance()->localStateDir() / "dump");
-    auto const workDirName = fs::path(fmt::format("contour-dump-{:%Y-%m-%d-%H-%M-%S}", chrono::system_clock::now()));
+    auto const workDirName = fs::path(std::format("contour-dump-{:%Y-%m-%d-%H-%M-%S}", chrono::system_clock::now()));
     auto const targetDir = targetBaseDir / workDirName;
     auto const latestDirName = fs::path("latest");
     // clang-format on
@@ -1111,7 +1118,7 @@ void TerminalDisplay::doDumpStateInternal()
         fs.close();
     }
 
-    enum class ImageBufferFormat
+    enum class ImageBufferFormat : uint8_t
     {
         RGBA,
         RGB,
@@ -1189,12 +1196,14 @@ void TerminalDisplay::resizeWindow(vtbackend::LineCount newLineCount, vtbackend:
     if (*newLineCount)
         requestedPageSize.lines = newLineCount;
 
-    auto const pixels = vtbackend::ImageSize {
-        boxed_cast<vtbackend::Width>(requestedPageSize.columns) * gridMetrics().cellSize.width,
-        boxed_cast<vtbackend::Height>(requestedPageSize.lines) * gridMetrics().cellSize.height
+    // Qt uses unscaled pixels, so we need to adjust the requested size to the actual content scale.
+    auto const unscaledCellSize = gridMetrics().cellSize / contentScale();
+    auto const unscaledViewSize = vtbackend::ImageSize {
+        unscaledCellSize.width * boxed_cast<vtbackend::Width>(requestedPageSize.columns),
+        unscaledCellSize.height * boxed_cast<vtbackend::Height>(requestedPageSize.lines)
     };
 
-    window()->resize(QSize(pixels.width.as<int>(), pixels.height.as<int>()));
+    window()->resize(QSize(unscaledViewSize.width.as<int>(), unscaledViewSize.height.as<int>()));
 }
 
 void TerminalDisplay::setFonts(vtrasterizer::FontDescriptions fontDescriptions)
@@ -1230,9 +1239,10 @@ bool TerminalDisplay::setPageSize(PageSize newPageSize)
     if (newPageSize == terminal().pageSize())
         return false;
 
-    auto const viewSize =
-        ImageSize { Width(*gridMetrics().cellSize.width * unbox<unsigned>(profile().terminalSize.columns)),
-                    Height(*gridMetrics().cellSize.width * unbox<unsigned>(profile().terminalSize.columns)) };
+    auto const viewSize = ImageSize {
+        Width(*gridMetrics().cellSize.width * unbox<unsigned>(profile().terminalSize.value().columns)),
+        Height(*gridMetrics().cellSize.width * unbox<unsigned>(profile().terminalSize.value().columns))
+    };
     _renderer->setPageSize(newPageSize);
     auto const l = scoped_lock { terminal() };
     terminal().resizeScreen(newPageSize, viewSize);
