@@ -1,4 +1,5 @@
 // SPDX-License-Identifier: Apache-2.0
+#include <contour/Config.h>
 #include <contour/TerminalSession.h>
 #include <contour/display/TerminalDisplay.h>
 #include <contour/helper.h>
@@ -58,8 +59,8 @@ namespace
         auto const cellSize = session.display()->cellSize();
         auto const dpr = session.contentScale();
 
-        auto const marginTop = static_cast<int>(session.profile().margins.vertical * dpr);
-        auto const marginLeft = static_cast<int>(session.profile().margins.horizontal * dpr);
+        auto const marginTop = static_cast<int>(unbox(session.profile().margins.value().vertical) * dpr);
+        auto const marginLeft = static_cast<int>(unbox(session.profile().margins.value().horizontal) * dpr);
 
         auto const sx = int(double(x) * dpr);
         auto const sy = int(double(y) * dpr);
@@ -74,7 +75,7 @@ namespace
     }
 
     PixelCoordinate makeMousePixelPosition(QHoverEvent* event,
-                                           config::TerminalProfile::WindowMargins margins,
+                                           config::WindowMargins margins,
                                            double dpr) noexcept
     {
 #if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
@@ -82,14 +83,14 @@ namespace
 #else
         auto const position = event->pos();
 #endif
-        auto const marginLeft = static_cast<int>(margins.horizontal * dpr);
-        auto const marginTop = static_cast<int>(margins.vertical * dpr);
+        auto const marginLeft = static_cast<int>(unbox(margins.horizontal) * dpr);
+        auto const marginTop = static_cast<int>(unbox(margins.vertical) * dpr);
         return PixelCoordinate { PixelCoordinate::X { int(double(position.x()) * dpr) - marginLeft },
                                  PixelCoordinate::Y { int(double(position.y()) * dpr) - marginTop } };
     }
 
     PixelCoordinate makeMousePixelPosition(QMouseEvent* event,
-                                           config::TerminalProfile::WindowMargins margins,
+                                           config::WindowMargins margins,
                                            double dpr) noexcept
     {
 #if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
@@ -97,14 +98,14 @@ namespace
 #else
         auto const position = QPointF { static_cast<qreal>(event->x()), static_cast<qreal>(event->y()) };
 #endif
-        auto const marginLeft = static_cast<int>(margins.horizontal * dpr);
-        auto const marginTop = static_cast<int>(margins.vertical * dpr);
+        auto const marginLeft = static_cast<int>(unbox(margins.horizontal) * dpr);
+        auto const marginTop = static_cast<int>(unbox(margins.vertical) * dpr);
         return PixelCoordinate { PixelCoordinate::X { int(double(position.x()) * dpr) - marginLeft },
                                  PixelCoordinate::Y { int(double(position.y()) * dpr) - marginTop } };
     }
 
     PixelCoordinate makeMousePixelPosition(QWheelEvent* event,
-                                           config::TerminalProfile::WindowMargins margins,
+                                           config::WindowMargins margins,
                                            double dpr) noexcept
     {
 #if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
@@ -112,8 +113,8 @@ namespace
 #else
         auto const position = event->posF();
 #endif
-        auto const marginLeft = static_cast<int>(margins.horizontal * dpr);
-        auto const marginTop = static_cast<int>(margins.vertical * dpr);
+        auto const marginLeft = static_cast<int>(unbox(margins.horizontal) * dpr);
+        auto const marginTop = static_cast<int>(unbox(margins.vertical) * dpr);
         return PixelCoordinate { PixelCoordinate::X { int(double(position.x()) * dpr) - marginLeft },
                                  PixelCoordinate::Y { int(double(position.y()) * dpr) - marginTop } };
     }
@@ -382,13 +383,13 @@ bool sendKeyEvent(QKeyEvent* event, vtbackend::KeyboardEventType eventType, Term
 
     if (key == Qt::Key_Backtab)
     {
-        session.sendCharEvent(U'\t', physicalKey, modifiers.with(Modifier::Shift), eventType, now);
+        session.sendKeyEvent(Key::Tab, modifiers.with(Modifier::Shift), eventType, now);
         event->accept();
         return true;
     }
 
 #if defined(__apple__)
-    if (0x20 <= key && key < 0x80 && (modifiers.alt() && session.profile().optionKeyAsAlt))
+    if (0x20 <= key && key < 0x80 && (modifiers.alt() && session.profile().optionKeyAsAlt.value()))
     {
         auto const ch = static_cast<char32_t>(modifiers.shift() ? std::toupper(key) : std::tolower(key));
         session.sendCharEvent(ch, physicalKey, modifiers, eventType, now);
@@ -409,8 +410,8 @@ bool sendKeyEvent(QKeyEvent* event, vtbackend::KeyboardEventType eventType, Term
         auto const codepoints = event->text().toUcs4();
         assert(codepoints.size() == 1);
 #if defined(__APPLE__)
-        // On OS/X the Alt-modifier does not seem to be passed to the terminal apps
-        // but rather remapped to whatever OS/X is mapping them to.
+        // On macOS the Alt-modifier does not seem to be passed to the terminal apps
+        // but rather remapped to whatever macOS is mapping them to.
         for (char32_t const ch: codepoints)
             session.sendCharEvent(ch, physicalKey, modifiers.without(Modifier::Alt), eventType, now);
 #else
@@ -442,7 +443,7 @@ void sendWheelEvent(QWheelEvent* event, TerminalSession& session)
         ((modifiers & Modifier::Alt) ? transposed(event->angleDelta()) : event->angleDelta()) / 8;
 
     auto const pixelPosition =
-        makeMousePixelPosition(event, session.profile().margins, session.contentScale());
+        makeMousePixelPosition(event, session.profile().margins.value(), session.contentScale());
 
     if (!pixelDelta.isNull())
     {
@@ -467,7 +468,7 @@ void sendMousePressEvent(QMouseEvent* event, TerminalSession& session)
     session.sendMousePressEvent(
         makeModifiers(event->modifiers()),
         makeMouseButton(event->button()),
-        makeMousePixelPosition(event, session.profile().margins, session.contentScale()));
+        makeMousePixelPosition(event, session.profile().margins.value(), session.contentScale()));
     event->accept();
 }
 
@@ -476,7 +477,7 @@ void sendMouseReleaseEvent(QMouseEvent* event, TerminalSession& session)
     session.sendMouseReleaseEvent(
         makeModifiers(event->modifiers()),
         makeMouseButton(event->button()),
-        makeMousePixelPosition(event, session.profile().margins, session.contentScale()));
+        makeMousePixelPosition(event, session.profile().margins.value(), session.contentScale()));
     event->accept();
 }
 
@@ -485,7 +486,7 @@ void sendMouseMoveEvent(QMouseEvent* event, TerminalSession& session)
     session.sendMouseMoveEvent(
         makeModifiers(event->modifiers()),
         makeMouseCellLocation(event->pos().x(), event->pos().y(), session),
-        makeMousePixelPosition(event, session.profile().margins, session.contentScale()));
+        makeMousePixelPosition(event, session.profile().margins.value(), session.contentScale()));
     event->accept();
 }
 
@@ -499,7 +500,7 @@ void sendMouseMoveEvent(QHoverEvent* event, TerminalSession& session)
     session.sendMouseMoveEvent(
         makeModifiers(event->modifiers()),
         makeMouseCellLocation(position.x(), position.y(), session),
-        makeMousePixelPosition(event, session.profile().margins, session.contentScale()));
+        makeMousePixelPosition(event, session.profile().margins.value(), session.contentScale()));
     event->accept();
 }
 
@@ -569,14 +570,14 @@ vtbackend::FontDef getFontDefinition(vtrasterizer::Renderer& renderer)
 vtrasterizer::PageMargin computeMargin(ImageSize cellSize,
                                        PageSize charCells,
                                        ImageSize displaySize,
-                                       config::TerminalProfile::WindowMargins minimumMargins) noexcept
+                                       config::WindowMargins minimumMargins) noexcept
 {
     auto const usedHeight = unbox(charCells.lines) * unbox(cellSize.height);
 
-    auto const topMargin = static_cast<int>(minimumMargins.vertical);
-    auto const bottomMargin = static_cast<int>(
-        std::min(unbox(displaySize.height) - usedHeight - topMargin, minimumMargins.vertical));
-    auto const leftMargin = static_cast<int>(minimumMargins.horizontal);
+    auto const topMargin = unbox<int>(minimumMargins.vertical);
+    auto const bottomMargin = unbox<int>(std::min(
+        config::VerticalMargin(unbox(displaySize.height) - usedHeight - topMargin), minimumMargins.vertical));
+    auto const leftMargin = unbox<int>(minimumMargins.horizontal);
 
     return { .left = leftMargin, .top = topMargin, .bottom = bottomMargin };
 }
@@ -614,17 +615,18 @@ void applyResize(vtbackend::ImageSize newPixelSize,
     auto const newPageSize =
         pageSizeForPixels(newPixelSize,
                           renderer.gridMetrics().cellSize,
-                          applyContentScale(session.profile().margins, session.contentScale()));
+                          applyContentScale(session.profile().margins.value(), session.contentScale()));
     vtbackend::Terminal& terminal = session.terminal();
     vtbackend::ImageSize const cellSize = renderer.gridMetrics().cellSize;
 
     if (renderer.hasRenderTarget())
         renderer.renderTarget().setRenderSize(newPixelSize);
     renderer.setPageSize(newPageSize);
-    renderer.setMargin(computeMargin(renderer.gridMetrics().cellSize,
-                                     newPageSize,
-                                     newPixelSize,
-                                     applyContentScale(session.profile().margins, session.contentScale())));
+    renderer.setMargin(
+        computeMargin(renderer.gridMetrics().cellSize,
+                      newPageSize,
+                      newPixelSize,
+                      applyContentScale(session.profile().margins.value(), session.contentScale())));
 
     if (oldPageSize.lines != newPageSize.lines)
         emit session.lineCountChanged(newPageSize.lines.as<int>());
@@ -636,7 +638,7 @@ void applyResize(vtbackend::ImageSize newPixelSize,
     displayLog()("Applying resize {}/{} pixels (margins {}) and {} -> {} cells.",
                  viewSize,
                  newPixelSize,
-                 session.profile().margins,
+                 session.profile().margins.value(),
                  terminal.pageSize(),
                  newPageSize);
 
