@@ -26,7 +26,9 @@
 #include FT_LCD_FILTER_H
 // clang-format on
 
-#include <fontconfig/fontconfig.h>
+#if __has_include(<fontconfig/fontconfig.h>)
+    #include <fontconfig/fontconfig.h>
+#endif
 
 #include <harfbuzz/hb-ft.h>
 #include <harfbuzz/hb.h>
@@ -90,7 +92,7 @@ struct hash<FontInfo>
     {
         auto fnv = crispy::fnv<char>();
         return size_t(
-            fnv(fnv(fd.path), to_string(fd.size.pt), fmt::format("{}", fd.weight))); // SSO should kick in.
+            fnv(fnv(fd.path), to_string(fd.size.pt), std::format("{}", fd.weight))); // SSO should kick in.
     }
 };
 } // namespace std
@@ -389,11 +391,12 @@ namespace
             gpos.presentation = presentation;
             result.emplace_back(gpos);
         }
+
         return crispy::none_of(result, glyphMissing);
     }
 } // namespace
 
-struct open_shaper::Private // {{{
+struct open_shaper::private_open_shaper // {{{
 {
     crispy::finally ftCleanup;
     FT_Library ft {};
@@ -476,7 +479,7 @@ struct open_shaper::Private // {{{
         return output;
     }
 
-    Private(DPI dpi, font_locator& locator):
+    private_open_shaper(DPI dpi, font_locator& locator):
         ftCleanup { [this]() {
             FT_Done_FreeType(ft);
         } },
@@ -551,7 +554,7 @@ struct open_shaper::Private // {{{
 }; // }}}
 
 open_shaper::open_shaper(DPI dpi, font_locator& locator):
-    _d(new Private(dpi, locator), [](Private* p) { delete p; })
+    _d(new private_open_shaper(dpi, locator), [](private_open_shaper* p) { delete p; })
 {
 }
 
