@@ -9,8 +9,7 @@
 
 #include <libunicode/convert.h>
 
-#include <fmt/format.h>
-
+#include <format>
 #include <optional>
 #include <set>
 #include <string>
@@ -20,7 +19,7 @@ namespace vtbackend
 {
 
 /// Mutualy exclusive mouse protocls.
-enum class MouseProtocol
+enum class MouseProtocol : uint16_t
 {
     /// Old X10 mouse protocol
     X10 = 9,
@@ -35,7 +34,7 @@ enum class MouseProtocol
 };
 
 // {{{ Modifier
-enum Modifier : unsigned
+enum Modifier : uint8_t
 {
     // NB: These values MUST match the values of the corresponding
     // the bit positions in the modifier mask in CSIu keyboard protocol.
@@ -62,7 +61,7 @@ std::string to_string(Modifiers modifier);
 
 // }}}
 // {{{ KeyInputEvent, Key
-enum class Key
+enum class Key : uint8_t
 {
     // function keys
     F1,
@@ -180,14 +179,14 @@ enum class Key
 
 std::string to_string(Key key);
 
-enum class KeyMode
+enum class KeyMode : uint8_t
 {
     Normal,
     Application
 };
 // }}}
 // {{{ Mouse
-enum class MouseButton
+enum class MouseButton : uint8_t
 {
     Left,
     Right,
@@ -201,7 +200,7 @@ enum class MouseButton
 
 std::string to_string(MouseButton button);
 
-enum class MouseTransport
+enum class MouseTransport : uint8_t
 {
     // CSI M Cb Cx Cy, with Cb, Cx, Cy incremented by 0x20
     Default,
@@ -217,7 +216,7 @@ enum class MouseTransport
 };
 // }}}
 
-enum class KeyboardEventType
+enum class KeyboardEventType : uint8_t
 {
     Press = 1,
     Repeat = 2,
@@ -286,9 +285,9 @@ class StandardKeyboardInputGenerator: public KeyboardInputGenerator
     void append(std::string_view sequence) { _pendingSequence += sequence; }
 
     template <typename... Args>
-    void append(fmt::format_string<Args...> const& text, Args... args)
+    void append(std::string_view const& text, Args... args)
     {
-        append(fmt::vformat(text, fmt::make_format_args(args...)));
+        append(std::vformat(text, std::make_format_args(args...)));
     }
 
     KeyMode _cursorKeysMode = KeyMode::Normal;
@@ -296,7 +295,7 @@ class StandardKeyboardInputGenerator: public KeyboardInputGenerator
     std::string _pendingSequence {};
 };
 
-enum class KeyboardEventFlag
+enum class KeyboardEventFlag : uint8_t
 {
     None = 0,
     DisambiguateEscapeCodes = 1,
@@ -401,7 +400,7 @@ class InputGenerator
     void setMouseTransport(MouseTransport mouseTransport);
     [[nodiscard]] MouseTransport mouseTransport() const noexcept { return _mouseTransport; }
 
-    enum class MouseWheelMode
+    enum class MouseWheelMode : uint8_t
     {
         // mouse wheel generates mouse wheel events as determined by mouse protocol + transport.
         Default,
@@ -472,7 +471,7 @@ class InputGenerator
         }
     }
 
-    enum class MouseEventType
+    enum class MouseEventType : uint8_t
     {
         Press,
         Drag,
@@ -552,9 +551,9 @@ inline std::string to_string(InputGenerator::MouseEventType value)
 // {{{ fmtlib custom formatter support
 
 template <>
-struct fmt::formatter<vtbackend::KeyboardEventType>: formatter<std::string_view>
+struct std::formatter<vtbackend::KeyboardEventType>: formatter<std::string_view>
 {
-    auto format(vtbackend::KeyboardEventType value, format_context& ctx) -> format_context::iterator
+    auto format(vtbackend::KeyboardEventType value, auto& ctx) const
     {
         string_view name;
         switch (value)
@@ -568,9 +567,9 @@ struct fmt::formatter<vtbackend::KeyboardEventType>: formatter<std::string_view>
 };
 
 template <>
-struct fmt::formatter<vtbackend::KeyboardEventFlag>: formatter<std::string_view>
+struct std::formatter<vtbackend::KeyboardEventFlag>: formatter<std::string_view>
 {
-    auto format(vtbackend::KeyboardEventFlag value, format_context& ctx) -> format_context::iterator
+    auto format(vtbackend::KeyboardEventFlag value, auto& ctx) const
     {
         string_view name;
         switch (value)
@@ -589,9 +588,9 @@ struct fmt::formatter<vtbackend::KeyboardEventFlag>: formatter<std::string_view>
 };
 
 template <>
-struct fmt::formatter<vtbackend::MouseProtocol>: formatter<std::string_view>
+struct std::formatter<vtbackend::MouseProtocol>: formatter<std::string_view>
 {
-    auto format(vtbackend::MouseProtocol value, format_context& ctx) -> format_context::iterator
+    auto format(vtbackend::MouseProtocol value, auto& ctx) const
     {
         string_view name;
         switch (value)
@@ -607,9 +606,9 @@ struct fmt::formatter<vtbackend::MouseProtocol>: formatter<std::string_view>
 };
 
 template <>
-struct fmt::formatter<vtbackend::Modifier>: formatter<std::string_view>
+struct std::formatter<vtbackend::Modifier>: formatter<std::string_view>
 {
-    auto format(vtbackend::Modifier value, format_context& ctx) -> format_context::iterator
+    auto format(vtbackend::Modifier value, auto& ctx) const
     {
         std::string_view name;
         switch (value)
@@ -629,10 +628,9 @@ struct fmt::formatter<vtbackend::Modifier>: formatter<std::string_view>
 };
 
 template <>
-struct fmt::formatter<vtbackend::InputGenerator::MouseWheelMode>: formatter<std::string_view>
+struct std::formatter<vtbackend::InputGenerator::MouseWheelMode>: formatter<std::string_view>
 {
-    auto format(vtbackend::InputGenerator::MouseWheelMode value, format_context& ctx)
-        -> format_context::iterator
+    auto format(vtbackend::InputGenerator::MouseWheelMode value, auto& ctx) const
     {
         string_view name;
         switch (value)
@@ -650,9 +648,9 @@ struct fmt::formatter<vtbackend::InputGenerator::MouseWheelMode>: formatter<std:
 };
 
 template <>
-struct fmt::formatter<vtbackend::KeyMode>: public formatter<std::string_view>
+struct std::formatter<vtbackend::KeyMode>: public formatter<std::string_view>
 {
-    auto format(vtbackend::KeyMode value, format_context& ctx) -> format_context::iterator
+    auto format(vtbackend::KeyMode value, auto& ctx) const
     {
         string_view name;
         switch (value)
@@ -665,9 +663,9 @@ struct fmt::formatter<vtbackend::KeyMode>: public formatter<std::string_view>
 };
 
 template <>
-struct fmt::formatter<vtbackend::MouseButton>: formatter<std::string_view>
+struct std::formatter<vtbackend::MouseButton>: formatter<std::string_view>
 {
-    auto format(vtbackend::MouseButton value, format_context& ctx) -> format_context::iterator
+    auto format(vtbackend::MouseButton value, auto& ctx) const
     {
         string_view name;
         switch (value)
@@ -686,9 +684,9 @@ struct fmt::formatter<vtbackend::MouseButton>: formatter<std::string_view>
 };
 
 template <>
-struct fmt::formatter<vtbackend::MouseTransport>: formatter<std::string_view>
+struct std::formatter<vtbackend::MouseTransport>: formatter<std::string_view>
 {
-    auto format(vtbackend::MouseTransport value, format_context& ctx) -> format_context::iterator
+    auto format(vtbackend::MouseTransport value, auto& ctx) const
     {
         string_view name;
         switch (value)
@@ -704,9 +702,9 @@ struct fmt::formatter<vtbackend::MouseTransport>: formatter<std::string_view>
 };
 
 template <>
-struct fmt::formatter<vtbackend::Key>: formatter<std::string_view>
+struct std::formatter<vtbackend::Key>: formatter<std::string_view>
 {
-    auto format(vtbackend::Key value, format_context& ctx) -> format_context::iterator
+    auto format(vtbackend::Key value, auto& ctx) const
     {
         string_view name;
         switch (value)
