@@ -4,6 +4,7 @@
 #include <vtbackend/CellUtil.h>
 #include <vtbackend/GraphicsAttributes.h>
 #include <vtbackend/Hyperlink.h>
+#include <vtbackend/cell/CellConcept.h>
 #include <vtbackend/primitives.h>
 
 #include <crispy/BufferObject.h>
@@ -52,7 +53,7 @@ struct TrivialLineBuffer
     HyperlinkId hyperlink {};
 
     ColumnCount usedColumns {};
-    crispy::BufferFragment<char> text {};
+    crispy::buffer_fragment<char> text {};
 
     void reset(GraphicsAttributes attributes) noexcept
     {
@@ -64,14 +65,14 @@ struct TrivialLineBuffer
     }
 };
 
-template <typename Cell>
+template <CellConcept Cell>
 using InflatedLineBuffer = std::vector<Cell>;
 
 /// Unpacks a TrivialLineBuffer into an InflatedLineBuffer<Cell>.
-template <typename Cell>
+template <CellConcept Cell>
 InflatedLineBuffer<Cell> inflate(TrivialLineBuffer const& input);
 
-template <typename Cell>
+template <CellConcept Cell>
 using LineStorage = std::variant<TrivialLineBuffer, InflatedLineBuffer<Cell>>;
 
 /**
@@ -80,7 +81,7 @@ using LineStorage = std::variant<TrivialLineBuffer, InflatedLineBuffer<Cell>>;
  * TODO: Use custom allocator for ensuring cache locality of Cells to sibling lines.
  * TODO: Make the line optimization work.
  */
-template <typename Cell>
+template <CellConcept Cell>
 class Line
 {
   public:
@@ -214,7 +215,8 @@ class Line
             return unbox<size_t>(column) >= trivialBuffer().text.size()
                    || trivialBuffer().text[column.as<size_t>()] == 0x20;
         }
-        return inflatedBuffer().at(unbox<size_t>(column)).empty();
+        auto const& cell = inflatedBuffer().at(unbox<size_t>(column));
+        return cell.empty() || (cell.codepointCount() == 1 && cell.codepoint(0) == 0x20);
     }
 
     [[nodiscard]] uint8_t cellWidthAt(ColumnOffset column) const noexcept
@@ -296,11 +298,15 @@ class Line
 
     void setBuffer(Storage buffer) noexcept { _storage = std::move(buffer); }
 
-    // Tests if the given text can be matched in this line at the exact given start column.
-    [[nodiscard]] bool matchTextAt(std::u32string_view text, ColumnOffset startColumn) const noexcept
+    // Tests if the given text can be matched in this line at the exact given start column, in sensetive
+    // or insensitive mode.
+    [[nodiscard]] bool matchTextAtWithSensetivityMode(std::u32string_view text,
+                                                      ColumnOffset startColumn,
+                                                      bool isCaseSensitive) const noexcept
     {
         if (isTrivialBuffer())
         {
+            assert(false);
             auto const u8Text = unicode::convert_to<char>(text);
             TrivialBuffer const& buffer = trivialBuffer();
             if (!buffer.usedColumns)
@@ -308,14 +314,15 @@ class Line
             auto const column = std::min(startColumn, boxed_cast<ColumnOffset>(buffer.usedColumns - 1));
             if (text.size() > static_cast<size_t>(column.value - buffer.usedColumns.value))
                 return false;
-            auto const resultIndex = buffer.text.view()
-                                         .substr(unbox<size_t>(column))
+
+            auto bufferCopyText = std::string(buffer.text.view());
+            std::transform(bufferCopyText.begin(), bufferCopyText.end(), bufferCopyText.begin(), ::tolower);
+            auto const resultIndex = bufferCopyText.substr(unbox<size_t>(column))
                                          .find(std::string_view(u8Text), unbox<size_t>(column));
             return resultIndex == 0;
         }
         else
         {
-            auto const u8Text = unicode::convert_to<char>(text);
             InflatedBuffer const& cells = inflatedBuffer();
             if (text.size() > unbox<size_t>(size()) - unbox<size_t>(startColumn))
                 return false;
@@ -323,7 +330,7 @@ class Line
             size_t i = 0;
             while (i < text.size())
             {
-                if (!CellUtil::beginsWith(text.substr(i), cells[baseColumn + i]))
+                if (!CellUtil::beginsWith(text.substr(i), cells[baseColumn + i], isCaseSensitive))
                     return false;
                 ++i;
             }
@@ -336,8 +343,14 @@ class Line
     // match is found at the right end of line it returns startColumn as it is and the partialMatchLength
     // is set equal to match found at the left end of line.
     [[nodiscard]] std::optional<SearchResult> search(std::u32string_view text,
-                                                     ColumnOffset startColumn) const noexcept
+                                                     ColumnOffset startColumn,
+                                                     bool isCaseSensitive) const noexcept
     {
+
+        auto matchTextAt = [&](auto text, auto baseColumn) {
+            return matchTextAtWithSensetivityMode(text, baseColumn, isCaseSensitive);
+        };
+
         if (isTrivialBuffer())
         {
             auto const u8Text = unicode::convert_to<char>(text);
@@ -381,8 +394,14 @@ class Line
     // match is found at the left end of line it returns startColumn as it is and the partialMatchLength
     // is set equal to match found at the left end of line.
     [[nodiscard]] std::optional<SearchResult> searchReverse(std::u32string_view text,
-                                                            ColumnOffset startColumn) const noexcept
+                                                            ColumnOffset startColumn,
+                                                            bool isCaseSensitive) const noexcept
     {
+
+        auto matchTextAt = [&](auto text, auto baseColumn) {
+            return matchTextAtWithSensetivityMode(text, baseColumn, isCaseSensitive);
+        };
+
         if (isTrivialBuffer())
         {
             auto const u8Text = unicode::convert_to<char>(text);
@@ -428,7 +447,7 @@ class Line
     LineFlags _flags;
 };
 
-template <typename Cell>
+template <CellConcept Cell>
 inline typename Line<Cell>::InflatedBuffer& Line<Cell>::inflatedBuffer()
 {
     if (auto trivialbuffer = std::get_if<TrivialBuffer>(&_storage))
@@ -436,7 +455,7 @@ inline typename Line<Cell>::InflatedBuffer& Line<Cell>::inflatedBuffer()
     return std::get<InflatedBuffer>(_storage);
 }
 
-template <typename Cell>
+template <CellConcept Cell>
 inline typename Line<Cell>::InflatedBuffer const& Line<Cell>::inflatedBuffer() const
 {
     return const_cast<Line<Cell>*>(this)->inflatedBuffer();
@@ -445,9 +464,9 @@ inline typename Line<Cell>::InflatedBuffer const& Line<Cell>::inflatedBuffer() c
 } // namespace vtbackend
 
 template <>
-struct fmt::formatter<vtbackend::LineFlags>: formatter<std::string>
+struct std::formatter<vtbackend::LineFlags>: formatter<std::string>
 {
-    auto format(const vtbackend::LineFlags flags, format_context& ctx) -> format_context::iterator
+    auto format(const vtbackend::LineFlags flags, auto& ctx) const
     {
         static const std::array<std::pair<vtbackend::LineFlags, std::string_view>, 3> nameMap = {
             std::pair { vtbackend::LineFlag::Wrappable, std::string_view("Wrappable") },
