@@ -15,8 +15,6 @@ using namespace std;
 namespace vtbackend
 {
 
-using crispy::beginsWith;
-
 namespace
 {
     ColumnCount graphemeClusterWidth(std::u32string_view cluster) noexcept
@@ -102,7 +100,7 @@ namespace
 
 } // namespace
 
-template <typename Cell>
+template <CellConcept Cell>
 RenderBufferBuilder<Cell>::RenderBufferBuilder(Terminal const& terminal,
                                                RenderBuffer& output,
                                                LineOffset base,
@@ -126,7 +124,7 @@ RenderBufferBuilder<Cell>::RenderBufferBuilder(Terminal const& terminal,
         output.cursor = renderCursor();
 }
 
-template <typename Cell>
+template <CellConcept Cell>
 optional<RenderCursor> RenderBufferBuilder<Cell>::renderCursor() const
 {
     if (!_cursorPosition || !_terminal->cursorCurrentlyVisible()
@@ -136,7 +134,7 @@ optional<RenderCursor> RenderBufferBuilder<Cell>::renderCursor() const
     // TODO: check if CursorStyle has changed, and update render context accordingly.
 
     auto constexpr InactiveCursorShape = CursorShape::Rectangle; // TODO configurable
-    auto const shape = _terminal->state().focused ? _terminal->cursorShape() : InactiveCursorShape;
+    auto const shape = _terminal->focused() ? _terminal->cursorShape() : InactiveCursorShape;
 
     auto const cursorScreenPosition =
         CellLocation { _baseLine + _cursorPosition->line
@@ -148,7 +146,7 @@ optional<RenderCursor> RenderBufferBuilder<Cell>::renderCursor() const
     return RenderCursor { cursorScreenPosition, shape, cellWidth };
 }
 
-template <typename Cell>
+template <CellConcept Cell>
 RenderCell RenderBufferBuilder<Cell>::makeRenderCellExplicit(ColorPalette const& colorPalette,
                                                              u32string graphemeCluster,
                                                              ColumnCount width,
@@ -171,7 +169,7 @@ RenderCell RenderBufferBuilder<Cell>::makeRenderCellExplicit(ColorPalette const&
     return renderCell;
 }
 
-template <typename Cell>
+template <CellConcept Cell>
 RenderCell RenderBufferBuilder<Cell>::makeRenderCellExplicit(ColorPalette const& colorPalette,
                                                              char32_t codepoint,
                                                              CellFlags flags,
@@ -194,7 +192,7 @@ RenderCell RenderBufferBuilder<Cell>::makeRenderCellExplicit(ColorPalette const&
     return renderCell;
 }
 
-template <typename Cell>
+template <CellConcept Cell>
 RenderCell RenderBufferBuilder<Cell>::makeRenderCell(ColorPalette const& colorPalette,
                                                      HyperlinkStorage const& hyperlinks,
                                                      Cell const& screenCell,
@@ -236,7 +234,7 @@ RenderCell RenderBufferBuilder<Cell>::makeRenderCell(ColorPalette const& colorPa
     return renderCell;
 }
 
-template <typename Cell>
+template <CellConcept Cell>
 RGBColorPair RenderBufferBuilder<Cell>::makeColorsForCell(CellLocation gridPosition,
                                                           CellFlags cellFlags,
                                                           Color foregroundColor,
@@ -271,7 +269,7 @@ RGBColorPair RenderBufferBuilder<Cell>::makeColorsForCell(CellLocation gridPosit
                       rapidBlink);
 }
 
-template <typename Cell>
+template <CellConcept Cell>
 RenderAttributes RenderBufferBuilder<Cell>::createRenderAttributes(
     CellLocation gridPosition, GraphicsAttributes graphicsAttributes) const noexcept
 {
@@ -288,7 +286,7 @@ RenderAttributes RenderBufferBuilder<Cell>::createRenderAttributes(
     return renderAttributes;
 }
 
-template <typename Cell>
+template <CellConcept Cell>
 RenderLine RenderBufferBuilder<Cell>::createRenderLine(TrivialLineBuffer const& lineBuffer,
                                                        LineOffset lineOffset) const
 {
@@ -305,13 +303,13 @@ RenderLine RenderBufferBuilder<Cell>::createRenderLine(TrivialLineBuffer const&
     return renderLine;
 }
 
-template <typename Cell>
+template <CellConcept Cell>
 bool RenderBufferBuilder<Cell>::gridLineContainsCursor(LineOffset lineOffset) const noexcept
 {
     if (_terminal->currentScreen().cursor().position.line == lineOffset)
         return true;
 
-    if (_cursorPosition && _terminal->state().inputHandler.mode() != ViMode::Insert)
+    if (_cursorPosition && _terminal->inputHandler().mode() != ViMode::Insert)
     {
         auto const viCursor = _terminal->viewport().translateGridToScreenCoordinate(_cursorPosition->line);
         if (viCursor == lineOffset)
@@ -321,11 +319,11 @@ bool RenderBufferBuilder<Cell>::gridLineContainsCursor(LineOffset lineOffset) co
     return false;
 }
 
-template <typename Cell>
+template <CellConcept Cell>
 void RenderBufferBuilder<Cell>::renderTrivialLine(TrivialLineBuffer const& lineBuffer, LineOffset lineOffset)
 {
     // if (lineBuffer.text.size())
-    //     fmt::print("Rendering trivial line {:2} 0..{}/{} ({} bytes): \"{}\"\n",
+    //     std::cout << std::format("Rendering trivial line {:2} 0..{}/{} ({} bytes): \"{}\"\n",
     //                lineOffset.value,
     //                lineBuffer.usedColumns,
     //                lineBuffer.displayWidth,
@@ -392,30 +390,24 @@ void RenderBufferBuilder<Cell>::renderTrivialLine(TrivialLineBuffer const& lineB
     _output->cells[backIndex].groupEnd = true;
 }
 
-template <typename Cell>
+template <CellConcept Cell>
 template <typename T>
 void RenderBufferBuilder<Cell>::matchSearchPattern(T const& cellText)
 {
     if (_highlightSearchMatches == HighlightSearchMatches::No)
         return;
 
-    auto const& searchMode = _terminal->state().searchMode;
-    if (searchMode.pattern.empty())
+    auto const& search = _terminal->search();
+    if (search.pattern.empty())
         return;
 
+    auto const searchText = u32string_view(search.pattern.data() + _searchPatternOffset,
+                                           search.pattern.size() - _searchPatternOffset);
+    auto const isCaseSensitive =
+        std::any_of(searchText.begin(), searchText.end(), [](auto ch) { return std::isupper(ch); });
+
     auto const isFullMatch = [&]() -> bool {
-        if constexpr (std::is_same_v<Cell, T>)
-        {
-            return !CellUtil::beginsWith(u32string_view(searchMode.pattern.data() + _searchPatternOffset,
-                                                        searchMode.pattern.size() - _searchPatternOffset),
-                                         cellText);
-        }
-        else
-        {
-            return crispy::beginsWith(u32string_view(searchMode.pattern.data() + _searchPatternOffset,
-                                                     searchMode.pattern.size() - _searchPatternOffset),
-                                      cellText);
-        }
+        return !CellUtil::beginsWith(searchText, cellText, isCaseSensitive);
     }();
 
     if (isFullMatch)
@@ -430,7 +422,7 @@ void RenderBufferBuilder<Cell>::matchSearchPattern(T const& cellText)
     else
         _searchPatternOffset += cellText.size();
 
-    if (_searchPatternOffset < searchMode.pattern.size())
+    if (_searchPatternOffset < search.pattern.size())
         return; // match incomplete
 
     // match complete
@@ -442,21 +434,21 @@ void RenderBufferBuilder<Cell>::matchSearchPattern(T const& cellText)
             _output->cells[offsetIntoFront].position,
             _output->cells.back().position,
         }
-            .contains(_terminal->viewport().translateGridToScreenCoordinate(
-                _terminal->state().viCommands.cursorPosition));
+            .contains(
+                _terminal->viewport().translateGridToScreenCoordinate(_terminal->normalModeCursorPosition()));
 
     auto highlightColors = [&]() -> CellRGBColorAndAlphaPair {
         // Oh yeah, this can be optimized :)
         if (isFocusedMatch)
         {
-            if (_terminal->state().searchMode.initiatedByDoubleClick)
+            if (_terminal->search().initiatedByDoubleClick)
                 return _terminal->colorPalette().wordHighlightCurrent;
             else
                 return _terminal->colorPalette().searchHighlightFocused;
         }
         else
         {
-            if (_terminal->state().searchMode.initiatedByDoubleClick)
+            if (_terminal->search().initiatedByDoubleClick)
                 return _terminal->colorPalette().wordHighlight;
             else
                 return _terminal->colorPalette().searchHighlight;
@@ -476,7 +468,7 @@ void RenderBufferBuilder<Cell>::matchSearchPattern(T const& cellText)
     _searchPatternOffset = 0;
 }
 
-template <typename Cell>
+template <CellConcept Cell>
 void RenderBufferBuilder<Cell>::startLine(LineOffset line) noexcept
 {
     _lineNr = line;
@@ -486,7 +478,7 @@ void RenderBufferBuilder<Cell>::startLine(LineOffset line) noexcept
     _useCursorlineColoring = isCursorLine(line);
 }
 
-template <typename Cell>
+template <CellConcept Cell>
 bool RenderBufferBuilder<Cell>::isCursorLine(LineOffset line) const noexcept
 {
     return _terminal->inputHandler().mode() != ViMode::Insert && _cursorPosition
@@ -496,7 +488,7 @@ bool RenderBufferBuilder<Cell>::isCursorLine(LineOffset line) const noexcept
                          .line;
 }
 
-template <typename Cell>
+template <CellConcept Cell>
 void RenderBufferBuilder<Cell>::endLine() noexcept
 {
     if (!_output->cells.empty())
@@ -505,7 +497,7 @@ void RenderBufferBuilder<Cell>::endLine() noexcept
     }
 }
 
-template <typename Cell>
+template <CellConcept Cell>
 ColumnCount RenderBufferBuilder<Cell>::renderUtf8Text(CellLocation screenPosition,
                                                       GraphicsAttributes textAttributes,
                                                       std::string_view text,
@@ -523,7 +515,7 @@ ColumnCount RenderBufferBuilder<Cell>::renderUtf8Text(CellLocation screenPositio
                                                 textAttributes.foregroundColor,
                                                 textAttributes.backgroundColor);
         auto const width = graphemeClusterWidth(graphemeCluster);
-        // fmt::print(" start {}, count {}, bytes {}, grapheme cluster \"{}\"\n",
+        // std::cout << std::format(" start {}, count {}, bytes {}, grapheme cluster \"{}\"\n",
         //            columnOffset,
         //            width,
         //            unicode::convert_to<char>(u32string_view(graphemeCluster)).size(),
@@ -566,7 +558,7 @@ ColumnCount RenderBufferBuilder<Cell>::renderUtf8Text(CellLocation screenPositio
     return columnCountRendered;
 }
 
-template <typename Cell>
+template <CellConcept Cell>
 bool RenderBufferBuilder<Cell>::tryRenderInputMethodEditor(CellLocation screenPosition,
                                                            CellLocation gridPosition)
 {
@@ -601,7 +593,7 @@ bool RenderBufferBuilder<Cell>::tryRenderInputMethodEditor(CellLocation screenPo
     return true;
 }
 
-template <typename Cell>
+template <CellConcept Cell>
 void RenderBufferBuilder<Cell>::renderCell(Cell const& screenCell, LineOffset line, ColumnOffset column)
 {
     auto const screenPosition = CellLocation { line, column };
@@ -616,13 +608,8 @@ void RenderBufferBuilder<Cell>::renderCell(Cell const& screenCell, LineOffset li
     _prevWidth = screenCell.width();
     _prevHasCursor = _cursorPosition && gridPosition == *_cursorPosition;
 
-    _output->cells.emplace_back(makeRenderCell(_terminal->colorPalette(),
-                                               _terminal->state().hyperlinks,
-                                               screenCell,
-                                               fg,
-                                               bg,
-                                               _baseLine + line,
-                                               column));
+    _output->cells.emplace_back(makeRenderCell(
+        _terminal->colorPalette(), _terminal->hyperlinks(), screenCell, fg, bg, _baseLine + line, column));
 
     if (column == ColumnOffset(0))
         _output->cells.back().groupStart = true;
