@@ -6,8 +6,7 @@
 
 #include <libunicode/ucd.h>
 
-#include <fmt/format.h>
-
+#include <format>
 #include <memory>
 
 namespace vtbackend
@@ -43,7 +42,7 @@ namespace
                || (192 <= codepoint && codepoint <= 255);
     }
 
-    enum class WordSkipClass
+    enum class WordSkipClass : uint8_t
     {
         Word,
         Keyword,
@@ -188,51 +187,29 @@ void ViCommands::searchDone()
 
 void ViCommands::searchCancel()
 {
-    _terminal->state().searchMode.pattern.clear();
+    _terminal->search().pattern.clear();
     _terminal->screenUpdated();
 }
 
 bool ViCommands::jumpToNextMatch(unsigned count)
 {
     for (unsigned i = 0; i < count; ++i)
-    {
-        auto startPosition = cursorPosition;
-        if (startPosition.column < boxed_cast<ColumnOffset>(_terminal->pageSize().columns))
-            startPosition.column++;
-        else if (cursorPosition.line < boxed_cast<LineOffset>(_terminal->pageSize().lines) - 1)
-        {
-            startPosition.line++;
-            startPosition.column = ColumnOffset(0);
-        }
-
-        auto const nextPosition = _terminal->search(startPosition);
-        if (!nextPosition)
+        if (auto const nextPosition = _terminal->searchNextMatch(cursorPosition))
+            moveCursorTo(nextPosition.value());
+        else
             return false;
 
-        moveCursorTo(nextPosition.value());
-    }
     return true;
 }
 
 bool ViCommands::jumpToPreviousMatch(unsigned count)
 {
     for (unsigned i = 0; i < count; ++i)
-    {
-        auto startPosition = cursorPosition;
-        if (startPosition.column != ColumnOffset(0))
-            startPosition.column--;
-        else if (cursorPosition.line > -boxed_cast<LineOffset>(_terminal->currentScreen().historyLineCount()))
-        {
-            startPosition.line--;
-            startPosition.column = boxed_cast<ColumnOffset>(_terminal->pageSize().columns) - 1;
-        }
-
-        auto const nextPosition = _terminal->searchReverse(startPosition);
-        if (!nextPosition)
+        if (auto const nextPosition = _terminal->searchPrevMatch(cursorPosition))
+            moveCursorTo(nextPosition.value());
+        else
             return false;
 
-        moveCursorTo(nextPosition.value());
-    }
     return true;
 }
 
@@ -261,7 +238,6 @@ void ViCommands::modeChanged(ViMode mode)
             _terminal->viewport().forceScrollToBottom();
             _terminal->clearSearch();
             _terminal->popStatusDisplay();
-            _terminal->screenUpdated();
             break;
         case ViMode::Normal:
             _lastCursorShape = _terminal->cursorShape();
@@ -273,7 +249,6 @@ void ViCommands::modeChanged(ViMode mode)
             if (_terminal->selectionAvailable())
                 _terminal->clearSelection();
             _terminal->pushStatusDisplay(StatusDisplayType::Indicator);
-            _terminal->screenUpdated();
             break;
         case ViMode::Visual:
             _terminal->setSelector(make_unique<LinearSelection>(
@@ -286,17 +261,16 @@ void ViCommands::modeChanged(ViMode mode)
                 _terminal->selectionHelper(), selectFrom, _terminal->selectionUpdatedHelper()));
             (void) _terminal->selector()->extend(cursorPosition);
             _terminal->pushStatusDisplay(StatusDisplayType::Indicator);
-            _terminal->screenUpdated();
             break;
         case ViMode::VisualBlock:
             _terminal->setSelector(make_unique<RectangularSelection>(
                 _terminal->selectionHelper(), selectFrom, _terminal->selectionUpdatedHelper()));
             (void) _terminal->selector()->extend(cursorPosition);
             _terminal->pushStatusDisplay(StatusDisplayType::Indicator);
-            _terminal->screenUpdated();
             break;
     }
 
+    _terminal->screenUpdated();
     _terminal->inputModeChanged(mode);
 }
 
@@ -345,7 +319,9 @@ void ViCommands::executeYank(ViMotion motion, unsigned count)
         }
         default: {
             auto const [from, to] = translateToCellRange(motion, count);
-            executeYank(from, to);
+            // motion is inclusive but for yank we want to exclude the last cell which is the first cell of
+            // the next word
+            executeYank(from, { to.line, to.column - 1 });
         }
         break;
     }
@@ -1004,6 +980,7 @@ CellLocation ViCommands::translateToCellLocation(ViMotion motion, unsigned count
                 prev = current;
                 current.column++;
             }
+
             return current;
         }
         case ViMotion::BigWordEndForward: // E
