@@ -10,8 +10,6 @@
 
 #include <variant>
 
-using std::nullopt;
-using std::optional;
 using std::pair;
 using std::vector;
 using namespace std::string_view_literals;
@@ -179,19 +177,20 @@ void ViInputHandler::registerAllCommands()
         char const key = theKey;
         ViOperator const op = viOperator;
 
-        auto const s1 = [](char ch) { return std::string(1, ch); };
-        registerCommand(ModeSelect::Normal, s1(toupper(key)), [this, op]() { _executor->execute(op, ViMotion::FullLine, count()); });
+        // operate on the full line, with yy or oo.
+        registerCommand(ModeSelect::Normal,
+                        std::format("{}{}", key, key),
+                        [this, op]() { _executor->execute(op, ViMotion::FullLine, count()); });
 
-        auto const s2 = [key](char ch) { return fmt::format("{}{}", key, ch); };
-        registerCommand(ModeSelect::Normal, s2(key), [this, op]() { _executor->execute(op, ViMotion::FullLine, count()); });
-        registerCommand(ModeSelect::Normal, s2('b'), [this, op]() { _executor->execute(op, ViMotion::WordBackward, count()); });
-        registerCommand(ModeSelect::Normal, s2('e'), [this, op]() { _executor->execute(op, ViMotion::WordEndForward, count()); });
-        registerCommand(ModeSelect::Normal, s2('w'), [this, op]() { _executor->execute(op, ViMotion::WordForward, count()); });
-        registerCommand(ModeSelect::Normal, s2('B'), [this, op]() { _executor->execute(op, ViMotion::BigWordBackward, count()); });
-        registerCommand(ModeSelect::Normal, s2('E'), [this, op]() { _executor->execute(op, ViMotion::BigWordEndForward, count()); });
-        registerCommand(ModeSelect::Normal, s2('W'), [this, op]() { _executor->execute(op, ViMotion::BigWordForward, count()); });
+        for (auto && [motionChars, motion]: MotionMappings)
+        {
+            // Passing motion as motion=motion (new variable) is yet another workaround for Clang 15 (Ubuntu) this time.
+            registerCommand(ModeSelect::Normal,
+                            std::format("{}{}", key, motionChars),
+                            [this, op, motion=motion]() { _executor->execute(op, motion, count()); });
+        }
 
-        auto const s3 = [key](char ch) { return fmt::format("{}{}.", key, ch); };
+        auto const s3 = [key](char ch) { return std::format("{}{}.", key, ch); };
         registerCommand(ModeSelect::Normal, s3('t'), [this, op]() { _executor->execute(op, ViMotion::TillBeforeCharRight, count(), _lastChar); });
         registerCommand(ModeSelect::Normal, s3('T'), [this, op]() { _executor->execute(op, ViMotion::TillAfterCharLeft, count(), _lastChar); });
         registerCommand(ModeSelect::Normal, s3('f'), [this, op]() { _executor->execute(op, ViMotion::ToCharRight, count(), _lastChar); });
@@ -204,10 +203,10 @@ void ViInputHandler::registerAllCommands()
         for (auto const& [objectChar, obj]: TextObjectMappings)
         {
             registerCommand(ModeSelect::Normal,
-                            fmt::format("y{}{}", scopeChar, objectChar),
+                            std::format("y{}{}", scopeChar, objectChar),
                             [this, scope = scope, obj = obj]() { _executor->yank(scope, obj); });
             registerCommand(ModeSelect::Normal,
-                            fmt::format("o{}{}", scopeChar, objectChar),
+                            std::format("o{}{}", scopeChar, objectChar),
                             [this, scope = scope, obj = obj]() { _executor->open(scope, obj); });
         }
     }
@@ -227,7 +226,7 @@ void ViInputHandler::registerAllCommands()
     for (auto const& [scopeChar, scope]: ScopeMappings)
         for (auto const& [objectChar, obj]: TextObjectMappings)
             registerCommand(ModeSelect::Visual,
-                            fmt::format("{}{}", scopeChar, objectChar),
+                            std::format("{}{}", scopeChar, objectChar),
                             [this, scope = scope, obj = obj]() { _executor->select(scope, obj); });
 }
 
@@ -245,6 +244,9 @@ void ViInputHandler::registerCommand(ModeSelect modes, std::string_view command,
 
     auto commandStr = crispy::replace(std::string(command.data(), command.size()), "<Space>", " ");
 
+    inputLog()(
+        "Registering command: {} in mode: {}", commandStr, modes == ModeSelect::Normal ? "Normal" : "Visual");
+
     switch (modes)
     {
         case ModeSelect::Normal: {
@@ -274,10 +276,9 @@ void ViInputHandler::appendModifierToPendingInput(Modifiers modifiers)
         _pendingInput += "C-";
 }
 
-bool ViInputHandler::handlePendingInput()
+void ViInputHandler::handlePendingInput()
 {
-    if (_pendingInput.empty())
-        return false;
+    assert(!_pendingInput.empty());
 
     auto constexpr TrieMapAllowWildcardDot = true;
 
@@ -285,7 +286,7 @@ bool ViInputHandler::handlePendingInput()
     auto const mappingResult = mapping.search(_pendingInput, TrieMapAllowWildcardDot);
     if (std::holds_alternative<crispy::exact_match<CommandHandler>>(mappingResult))
     {
-        inputLog()("Executing handler for: {}{}", _count ? fmt::format("{} ", _count) : "", _pendingInput);
+        inputLog()("Executing handler for: {}{}", _count ? std::format("{} ", _count) : "", _pendingInput);
         _lastChar =
             unicode::convert_to<char32_t>(std::string_view(_pendingInput.data(), _pendingInput.size()))
                 .back();
@@ -301,8 +302,6 @@ bool ViInputHandler::handlePendingInput()
     {
         inputLog()("Incomplete input: {}", _pendingInput);
     }
-
-    return true;
 }
 
 void ViInputHandler::clearPendingInput()
@@ -323,8 +322,11 @@ void ViInputHandler::setMode(ViMode theMode)
     _executor->modeChanged(theMode);
 }
 
-bool ViInputHandler::sendKeyPressEvent(Key key, Modifiers modifiers)
+Handled ViInputHandler::sendKeyPressEvent(Key key, Modifiers modifiers, KeyboardEventType eventType)
 {
+    if (eventType == KeyboardEventType::Release)
+        return Handled { true };
+
     if (_searchEditMode != SearchEditMode::Disabled)
     {
         // TODO: support cursor movements.
@@ -335,16 +337,16 @@ bool ViInputHandler::sendKeyPressEvent(Key key, Modifiers modifiers)
             case Key::Escape: return handleSearchEditor('\x1B', modifiers);
             default: break;
         }
-        return true;
+        return Handled { true };
     }
 
     // clang-format off
     switch (_viMode)
     {
         case ViMode::Insert:
-            return false;
-        case ViMode::Normal:
-            break;
+            // In insert mode we do not handle any key events here.
+            // The terminal will handle them and send them to the application.
+            return Handled { false };
         case ViMode::Visual:
         case ViMode::VisualLine:
         case ViMode::VisualBlock:
@@ -352,13 +354,16 @@ bool ViInputHandler::sendKeyPressEvent(Key key, Modifiers modifiers)
             {
                 clearPendingInput();
                 setMode(ViMode::Normal);
-                return true;
+                return Handled { false };
             }
+            [[fallthrough]];
+        case ViMode::Normal:
+            // We keep on handling key events below.
             break;
     }
     // clang-format on
 
-    auto const charMappings = std::array<std::pair<Key, char32_t>, 10> { {
+    auto const charMappings = std::array<std::pair<Key, char32_t>, 12> { {
         { Key::Numpad_0, '0' },
         { Key::Numpad_1, '1' },
         { Key::Numpad_2, '2' },
@@ -369,14 +374,16 @@ bool ViInputHandler::sendKeyPressEvent(Key key, Modifiers modifiers)
         { Key::Numpad_7, '7' },
         { Key::Numpad_8, '8' },
         { Key::Numpad_9, '9' },
+        { Key::Backspace, '\b' },
+        { Key::Enter, '\n' },
     } };
 
     for (auto const& [mappedKey, mappedText]: charMappings)
         if (key == mappedKey)
-            return sendCharPressEvent(mappedText, modifiers);
+            return sendCharPressEvent(mappedText, modifiers, eventType);
 
     if (modifiers.any())
-        return true;
+        return Handled { true };
 
     auto const keyMappings = std::array<std::pair<Key, std::string_view>, 10> { {
         { Key::DownArrow, "<Down>" },
@@ -401,10 +408,11 @@ bool ViInputHandler::sendKeyPressEvent(Key key, Modifiers modifiers)
     if (_pendingInput.empty())
     {
         errorLog()("ViInputHandler: Unhandled key: {} ({})", key, modifiers);
-        return false;
+        return Handled { true };
     }
 
-    return handlePendingInput();
+    handlePendingInput();
+    return Handled { true };
 }
 
 void ViInputHandler::startSearchExternally()
@@ -423,7 +431,7 @@ void ViInputHandler::startSearchExternally()
     }
 }
 
-bool ViInputHandler::handleSearchEditor(char32_t ch, Modifiers modifiers)
+Handled ViInputHandler::handleSearchEditor(char32_t ch, Modifiers modifiers)
 {
     assert(_searchEditMode != SearchEditMode::Disabled);
 
@@ -437,7 +445,7 @@ bool ViInputHandler::handleSearchEditor(char32_t ch, Modifiers modifiers)
             _executor->searchCancel();
             break;
         case '\x0D'_key:
-            if (_searchEditMode == SearchEditMode::ExternallyEnabled)
+            if (_settings.fromSearchIntoInsertMode && _searchEditMode == SearchEditMode::ExternallyEnabled)
                 setMode(ViMode::Insert);
             _searchEditMode = SearchEditMode::Disabled;
             _executor->searchDone();
@@ -467,26 +475,29 @@ bool ViInputHandler::handleSearchEditor(char32_t ch, Modifiers modifiers)
                            (unsigned) ch);
     }
 
-    return true;
+    return Handled { true };
 }
 
-bool ViInputHandler::sendCharPressEvent(char32_t ch, Modifiers modifiers)
+Handled ViInputHandler::sendCharPressEvent(char32_t ch, Modifiers modifiers, KeyboardEventType eventType)
 {
+    if (eventType == KeyboardEventType::Release)
+        return Handled { true };
+
     if (_searchEditMode != SearchEditMode::Disabled)
         return handleSearchEditor(ch, modifiers);
 
     if (_viMode == ViMode::Insert)
-        return false;
+        return Handled { false };
 
     if (ch == 033 && modifiers.none())
     {
         clearPendingInput();
         setMode(ViMode::Normal);
-        return true;
+        return Handled { true };
     }
 
     if (parseCount(ch, modifiers))
-        return true;
+        return Handled { true };
 
     appendModifierToPendingInput(ch > 0x20 ? modifiers.without(Modifier::Shift) : modifiers);
 
@@ -502,13 +513,15 @@ bool ViInputHandler::sendCharPressEvent(char32_t ch, Modifiers modifiers)
     if (_pendingInput.empty())
     {
         errorLog()("ViInputHandler: Unhandled char: {} ({})", static_cast<uint32_t>(ch), modifiers);
-        return false;
+        return Handled { false };
     }
 
-    if (handlePendingInput())
-        return true;
+    if (!_pendingInput.empty())
+        handlePendingInput();
+    else
+        errorLog()("ViInputHandler: Unhandled character: {} ({})", (unsigned) ch, modifiers);
 
-    return false;
+    return Handled { true };
 }
 
 bool ViInputHandler::parseCount(char32_t ch, Modifiers modifiers)
@@ -551,4 +564,9 @@ void ViInputHandler::toggleMode(ViMode newMode)
     setMode(newMode != _viMode ? newMode : ViMode::Normal);
 }
 
+void ViInputHandler::setSearchModeSwitch(bool enabled)
+{
+    _settings.fromSearchIntoInsertMode = enabled;
+}
+
 } // namespace vtbackend
