@@ -4,8 +4,10 @@
 #include <vtpty/ImageSize.h>
 #include <vtpty/PageSize.h>
 
+#include <cstdint>
 #include <limits>
 #include <ostream>
+#include <string>
 #include <variant>
 
 #include <boxed-cpp/boxed.hpp>
@@ -17,6 +19,16 @@
 namespace vtbackend
 {
 
+struct FontDef
+{
+    double size;
+    std::string regular;
+    std::string bold;
+    std::string italic;
+    std::string boldItalic;
+    std::string emoji;
+};
+
 using LineCount = vtpty::LineCount;
 using ColumnCount = vtpty::ColumnCount;
 using PageSize = vtpty::PageSize;
@@ -106,6 +118,7 @@ struct PixelCoordinate
     Y y {};
 };
 
+// {{{ CellLocation and related types
 struct [[nodiscard]] CellLocation
 {
     LineOffset line {};
@@ -134,7 +147,7 @@ struct [[nodiscard]] CellLocation
 
 inline std::ostream& operator<<(std::ostream& os, CellLocation coord)
 {
-    return os << fmt::format("({}, {})", coord.line, coord.column);
+    return os << std::format("({}, {})", coord.line, coord.column);
 }
 
 inline CellLocation operator+(CellLocation a, CellLocation b) noexcept
@@ -217,7 +230,6 @@ struct ColumnRange
         return line == location.line && fromColumn <= location.column && location.column <= toColumn;
     }
 };
-
 // }}}
 // {{{ Range
 
@@ -353,12 +365,15 @@ struct GridSize
 
         constexpr Offset makeOffset(int offset) noexcept
         {
-            return Offset { LineOffset(offset / *_width), ColumnOffset(offset % *_width) };
+            return Offset { LineOffset(offset / unbox(_width)), ColumnOffset(offset % unbox(_width)) };
         }
     };
 
     [[nodiscard]] constexpr iterator begin() const noexcept { return iterator { columns, 0 }; }
-    [[nodiscard]] constexpr iterator end() const noexcept { return iterator { columns, *columns * *lines }; }
+    [[nodiscard]] constexpr iterator end() const noexcept
+    {
+        return iterator { columns, unbox(columns) * unbox(lines) };
+    }
 };
 
 constexpr CellLocation operator+(CellLocation a, GridSize::Offset b) noexcept
@@ -436,16 +451,16 @@ constexpr ColumnOffset& operator-=(ColumnOffset& a, ColumnCount b) noexcept
 }
 // }}}
 
-enum class HighlightSearchMatches
+enum class HighlightSearchMatches : uint8_t
 {
     No,
     Yes
 };
 
-enum class ScreenType
+enum class ScreenType : uint8_t
 {
     Primary = 0,
-    Alternate = 1
+    Alternate = 1,
 };
 
 // TODO: Maybe make boxed.h into its own C++ github repo?
@@ -460,13 +475,13 @@ enum class ScreenType
 // - PhysicalCoordinate
 // - ScrollbackCoordinate
 
-enum class CursorDisplay
+enum class CursorDisplay : uint8_t
 {
     Steady,
     Blink
 };
 
-enum class CursorShape
+enum class CursorShape : uint8_t
 {
     Block,
     Rectangle,
@@ -476,13 +491,13 @@ enum class CursorShape
 
 CursorShape makeCursorShape(std::string const& name);
 
-enum class ControlTransmissionMode
+enum class ControlTransmissionMode : uint8_t
 {
     S7C1T, // 7-bit controls
     S8C1T, // 8-bit controls
 };
 
-enum class GraphicsRendition
+enum class GraphicsRendition : uint8_t
 {
     Reset = 0, //!< Reset any rendition (style as well as foreground / background coloring).
 
@@ -514,7 +529,7 @@ enum class GraphicsRendition
     NoOverline = 55,      //!< Reverses Overline.
 };
 
-enum class StatusDisplayType
+enum class StatusDisplayType : uint8_t
 {
     None,
     Indicator,
@@ -522,7 +537,7 @@ enum class StatusDisplayType
 };
 
 // Mandates the position to show the statusline at.
-enum class StatusDisplayPosition
+enum class StatusDisplayPosition : uint8_t
 {
     // The status line is classically shown at the bottom of the render target.
     Bottom,
@@ -532,7 +547,7 @@ enum class StatusDisplayPosition
 };
 
 // Selects whether the terminal sends data to the main display or the status line.
-enum class ActiveStatusDisplay
+enum class ActiveStatusDisplay : uint8_t
 {
     // Selects the main display. The terminal sends data to the main display only.
     Main,
@@ -543,7 +558,7 @@ enum class ActiveStatusDisplay
     IndicatorStatusLine,
 };
 
-enum class AnsiMode
+enum class AnsiMode : uint8_t
 {
     KeyboardAction = 2,    // KAM
     Insert = 4,            // IRM
@@ -551,7 +566,7 @@ enum class AnsiMode
     AutomaticNewLine = 20, // LNM
 };
 
-enum class DECMode
+enum class DECMode : std::uint16_t
 {
     UseApplicationCursorKeys,
     DesignateCharsetUSASCII,
@@ -666,7 +681,7 @@ enum class DECMode
 };
 
 /// OSC color-setting related commands that can be grouped into one
-enum class DynamicColorName
+enum class DynamicColorName : uint8_t
 {
     DefaultForegroundColor,
     DefaultBackgroundColor,
@@ -677,7 +692,7 @@ enum class DynamicColorName
     HighlightBackgroundColor,
 };
 
-enum class ViMode
+enum class ViMode : uint8_t
 {
     /// Vi-like normal-mode.
     Normal, // <Escape>, <C-[>
@@ -871,9 +886,9 @@ struct numeric_limits<vtbackend::CursorShape>
 
 // {{{ fmt formatter
 template <>
-struct fmt::formatter<vtbackend::CursorShape>: formatter<std::string_view>
+struct std::formatter<vtbackend::CursorShape>: formatter<std::string_view>
 {
-    auto format(vtbackend::CursorShape value, format_context& ctx) -> format_context::iterator
+    auto format(vtbackend::CursorShape value, auto& ctx) const
     {
         string_view name;
         switch (value)
@@ -888,36 +903,36 @@ struct fmt::formatter<vtbackend::CursorShape>: formatter<std::string_view>
 };
 
 template <>
-struct fmt::formatter<vtbackend::CellLocation>: formatter<std::string>
+struct std::formatter<vtbackend::CellLocation>: formatter<std::string>
 {
-    auto format(vtbackend::CellLocation coord, format_context& ctx) -> format_context::iterator
+    auto format(vtbackend::CellLocation coord, auto& ctx) const
     {
-        return formatter<std::string>::format(fmt::format("({}, {})", coord.line, coord.column), ctx);
+        return formatter<std::string>::format(std::format("({}, {})", coord.line, coord.column), ctx);
     }
 };
 
 template <>
-struct fmt::formatter<vtbackend::PageSize>: formatter<std::string>
+struct std::formatter<vtbackend::PageSize>: formatter<std::string>
 {
-    auto format(vtbackend::PageSize value, format_context& ctx) -> format_context::iterator
+    auto format(vtbackend::PageSize value, auto& ctx) const
     {
-        return formatter<std::string>::format(fmt::format("{}x{}", value.columns, value.lines), ctx);
+        return formatter<std::string>::format(std::format("{}x{}", value.columns, value.lines), ctx);
     }
 };
 
 template <>
-struct fmt::formatter<vtbackend::GridSize>: formatter<std::string>
+struct std::formatter<vtbackend::GridSize>: formatter<std::string>
 {
-    auto format(vtbackend::GridSize value, format_context& ctx) -> format_context::iterator
+    auto format(vtbackend::GridSize value, auto& ctx) const
     {
-        return formatter<std::string>::format(fmt::format("{}x{}", value.columns, value.lines), ctx);
+        return formatter<std::string>::format(std::format("{}x{}", value.columns, value.lines), ctx);
     }
 };
 
 template <>
-struct fmt::formatter<vtbackend::ScreenType>: formatter<std::string_view>
+struct std::formatter<vtbackend::ScreenType>: formatter<std::string_view>
 {
-    auto format(const vtbackend::ScreenType value, format_context& ctx) -> format_context::iterator
+    auto format(const vtbackend::ScreenType value, auto& ctx) const
     {
         string_view name;
         switch (value)
@@ -930,18 +945,18 @@ struct fmt::formatter<vtbackend::ScreenType>: formatter<std::string_view>
 };
 
 template <>
-struct fmt::formatter<vtbackend::PixelCoordinate>: formatter<std::string>
+struct std::formatter<vtbackend::PixelCoordinate>: formatter<std::string>
 {
-    auto format(const vtbackend::PixelCoordinate coord, format_context& ctx) -> format_context::iterator
+    auto format(const vtbackend::PixelCoordinate coord, auto& ctx) const
     {
-        return formatter<std::string>::format(fmt::format("{}:{}", coord.x.value, coord.y.value), ctx);
+        return formatter<std::string>::format(std::format("{}:{}", coord.x.value, coord.y.value), ctx);
     }
 };
 
 template <>
-struct fmt::formatter<vtbackend::ViMode>: formatter<std::string_view>
+struct std::formatter<vtbackend::ViMode>: formatter<std::string_view>
 {
-    auto format(vtbackend::ViMode mode, format_context& ctx) -> format_context::iterator
+    auto format(vtbackend::ViMode mode, auto& ctx) const
     {
         using vtbackend::ViMode;
         string_view name;
