@@ -186,15 +186,15 @@ std::string SshHostConfig::toString() const
         }
     };
     if (!username.empty())
-        result += fmt::format("{}@", username);
+        result += std::format("{}@", username);
     if (!hostname.empty())
     {
         if (port)
         {
             if (hostname.find(':') != std::string::npos)
-                result += fmt::format("[{}]:{}"sv, hostname, port);
+                result += std::format("[{}]:{}"sv, hostname, port);
             else
-                result += fmt::format("{}:{}"sv, hostname, port);
+                result += std::format("{}:{}"sv, hostname, port);
         }
         else
             result += hostname;
@@ -203,15 +203,15 @@ std::string SshHostConfig::toString() const
         result += "*:" + std::to_string(port);
 
     if (!privateKeyFile.empty())
-        add(fmt::format("private key: {}", privateKeyFile.string()));
+        add(std::format("private key: {}", privateKeyFile.string()));
 
     if (!publicKeyFile.empty())
-        add(fmt::format("public key: {}", privateKeyFile.string()));
+        add(std::format("public key: {}", privateKeyFile.string()));
 
     if (!knownHostsFile.empty())
-        add(fmt::format("known hosts: {}", knownHostsFile.string()));
+        add(std::format("known hosts: {}", knownHostsFile.string()));
 
-    add(fmt::format("ForwardAgent: {}", forwardAgent ? "Yes" : "No"));
+    add(std::format("ForwardAgent: {}", forwardAgent ? "Yes" : "No"));
 
     return result;
 }
@@ -222,20 +222,20 @@ std::string SshHostConfig::toConfigString(std::string const& host) const
     auto const prefix = host.empty() ? ""sv : "  "sv;
 
     if (!host.empty())
-        result += fmt::format("Host {}\n", host);
+        result += std::format("Host {}\n", host);
 
     if (!hostname.empty())
-        result += fmt::format("{}HostName {}\n", prefix, hostname);
+        result += std::format("{}HostName {}\n", prefix, hostname);
     if (port != 22)
-        result += fmt::format("{}Port {}\n", prefix, port);
+        result += std::format("{}Port {}\n", prefix, port);
     if (!username.empty())
-        result += fmt::format("{}User {}\n", prefix, username);
+        result += std::format("{}User {}\n", prefix, username);
     if (!privateKeyFile.empty())
-        result += fmt::format("{}IdentityFile {}\n", prefix, privateKeyFile.string());
+        result += std::format("{}IdentityFile {}\n", prefix, privateKeyFile.string());
     if (!knownHostsFile.empty())
-        result += fmt::format("{}KnownHostsFile {}\n", prefix, knownHostsFile.string());
-    result += fmt::format("{}ForwardAgent {}\n", prefix, forwardAgent);
-    result += fmt::format("\n");
+        result += std::format("{}KnownHostsFile {}\n", prefix, knownHostsFile.string());
+    result += std::format("{}ForwardAgent {}\n", prefix, forwardAgent);
+    result += std::format("\n");
     return result;
 }
 
@@ -450,7 +450,7 @@ void SshSession::processState()
             }
             case State::AuthenticatePasswordStart: {
                 setState(State::AuthenticatePasswordWaitForInput);
-                injectRead(fmt::format("\U0001F511 Username: {}\r\n", _config.username));
+                injectRead(std::format("\U0001F511 Username: {}\r\n", _config.username));
                 injectRead("\U0001F511 Password: ");
                 [[fallthrough]];
             }
@@ -612,9 +612,10 @@ void SshSession::processState()
 void SshSession::start()
 {
     if (_config.port == 22)
-        logInfo("Starting SSH session to host: {}@{}", _config.username, _config.hostname);
+        logInfoWithInject("Starting SSH session to host: {}@{}", _config.username, _config.hostname);
     else
-        logInfo("Starting SSH session to host: {}@{}:{}", _config.username, _config.hostname, _config.port);
+        logInfoWithInject(
+            "Starting SSH session to host: {}@{}:{}", _config.username, _config.hostname, _config.port);
 
     assert(_state == State::Initial);
     // auto const _ = std::lock_guard { _mutex };
@@ -688,9 +689,9 @@ void SshSession::waitForClosed()
     _closedCV.wait(lock, [this]() -> bool { return isClosed(); });
 }
 
-SshSession::ReadResult SshSession::read(crispy::buffer_object<char>& storage,
-                                        std::optional<std::chrono::milliseconds> timeout,
-                                        size_t size)
+std::optional<SshSession::ReadResult> SshSession::read(crispy::buffer_object<char>& storage,
+                                                       std::optional<std::chrono::milliseconds> timeout,
+                                                       size_t size)
 {
     auto injectLock = std::unique_lock { _injectMutex };
     _injectCV.wait(injectLock, [this]() { return _state == State::Operational || !_injectedRead.empty(); });
@@ -700,7 +701,8 @@ SshSession::ReadResult SshSession::read(crispy::buffer_object<char>& storage,
         auto const nread = std::min(size, _injectedRead.size());
         std::copy_n(_injectedRead.begin(), nread, storage.hotEnd());
         _injectedRead.erase(0, nread);
-        return std::tuple { std::string_view { storage.hotEnd(), nread }, false };
+        return ReadResult { .data = std::string_view { storage.hotEnd(), nread },
+                            .fromStdoutFastPipe = false };
     }
 
     if (_state == State::AuthenticatePasswordWaitForInput)
@@ -744,7 +746,7 @@ SshSession::ReadResult SshSession::read(crispy::buffer_object<char>& storage,
         ptyInLog()(
             "{} received: \"{}\"", "ssh", crispy::escape(target.data(), target.data() + target.size()));
 
-    return std::tuple { target, isStdFastPipe };
+    return ReadResult { .data = target, .fromStdoutFastPipe = isStdFastPipe };
 }
 
 void SshSession::wakeupReader()
@@ -897,16 +899,25 @@ void SshSession::injectRead(std::string_view buf)
     _injectCV.notify_all();
 }
 
+void SshSession::logInject(std::string_view message) const
+{
+    const_cast<SshSession*>(this)->injectRead(std::format("\U0001F511 \033[1;33m{}\033[m\r\n", message));
+}
+
 void SshSession::logInfo(std::string_view message) const
 {
     sshLog()("{}", message);
-    const_cast<SshSession*>(this)->injectRead(fmt::format("\U0001F511 \033[1;33m{}\033[m\r\n", message));
+}
+
+void SshSession::logInfoWithInject(std::string_view message) const
+{
+    logInfo(message);
+    logInject(message);
 }
 
 void SshSession::logError(std::string_view message) const
 {
     errorLog()("{}", message);
-    const_cast<SshSession*>(this)->injectRead(fmt::format("\U0001F511 \033[1;31m{}\033[m\r\n", message));
 }
 
 bool SshSession::connect(std::string_view host, int port)
@@ -922,11 +933,12 @@ bool SshSession::connect(std::string_view host, int port)
 
     try
     {
-        auto hints = addrinfo {
-            .ai_family = PF_UNSPEC,
-            .ai_socktype = SOCK_STREAM,
-            .ai_protocol = IPPROTO_TCP,
-        };
+        auto hints = addrinfo {};
+        hints.ai_flags = AI_ADDRCONFIG;
+        hints.ai_family = PF_UNSPEC;
+        hints.ai_socktype = SOCK_STREAM;
+        hints.ai_protocol = IPPROTO_TCP;
+
         addrinfo* addrList = nullptr;
         if (auto const rc = getaddrinfo(host.data(), nullptr, &hints, &addrList); rc != 0)
         {
@@ -965,11 +977,11 @@ bool SshSession::connect(std::string_view host, int port)
             if (::connect(_p->sshSocket, addrEntry->ai_addr, addrEntry->ai_addrlen) == 0)
             {
                 auto const addrAndPort =
-                    port == 22 ? std::string(addrStr) : fmt::format("{}:{}", addrStr, port);
+                    port == 22 ? std::string(addrStr) : std::format("{}:{}", addrStr, port);
                 if (host != addrStr)
-                    logInfo("Connected to {} ({})", host, addrAndPort);
+                    logInfoWithInject("Connected to {} ({})", host, addrAndPort);
                 else
-                    logInfo("Connected to {}", addrAndPort);
+                    logInfoWithInject("Connected to {}", addrAndPort);
                 return true;
             }
 
@@ -1061,7 +1073,7 @@ bool SshSession::verifyHostKey()
         case LIBSSH2_KNOWNHOST_CHECK_NOTFOUND: {
             // TODO: Ask user whether to add host key to known_hosts file
             auto const comment =
-                fmt::format("{}@{}:{} (added by Contour)", _config.username, _config.hostname, _config.port);
+                std::format("{}@{}:{} (added by Contour)", _config.username, _config.hostname, _config.port);
             auto const typeMask = LIBSSH2_KNOWNHOST_TYPE_PLAIN | LIBSSH2_KNOWNHOST_KEYENC_RAW | knownhostType;
             libssh2_knownhost_addc(knownHosts,
                                    _config.hostname.c_str(),
@@ -1135,21 +1147,23 @@ int SshSession::waitForSocket(std::optional<std::chrono::milliseconds> timeout)
     if (dir & LIBSSH2_SESSION_BLOCK_INBOUND)
     {
         readfd = &fd;
-        fmt::print("({}) SshSession: waiting for socket to become readable\n", crispy::threadName());
+        std::cout << std::format("({}) SshSession: waiting for socket to become readable\n",
+                                 crispy::threadName());
     }
 
     if (dir & LIBSSH2_SESSION_BLOCK_OUTBOUND)
     {
         writefd = &fd;
-        fmt::print("({}) SshSession: waiting for socket to become readable\n", crispy::threadName());
+        std::cout << std::format("({}) SshSession: waiting for socket to become readable\n",
+                                 crispy::threadName());
     }
 
     auto const rc = ::select((int) (_p->sshSocket + 1), readfd, writefd, nullptr, timeout ? &tv : nullptr);
-    fmt::print("({}) SshSession: select() returned {}{}{}\n",
-               crispy::threadName(),
-               rc,
-               readfd && FD_ISSET(_p->sshSocket, readfd) ? " [readable]" : "",
-               writefd && FD_ISSET(_p->sshSocket, writefd) ? " [writable]" : "");
+    std::cout << std::format("({}) SshSession: select() returned {}{}{}\n",
+                             crispy::threadName(),
+                             rc,
+                             readfd && FD_ISSET(_p->sshSocket, readfd) ? " [readable]" : "",
+                             writefd && FD_ISSET(_p->sshSocket, writefd) ? " [writable]" : "");
     return rc;
 #else
     crispy::ignore_unused(timeout);
@@ -1195,7 +1209,7 @@ void SshSession::authenticateWithPrivateKey()
         return;
     }
 
-    logInfo("Successfully authenticated with private key.");
+    logInfoWithInject("Successfully authenticated with private key.");
 
     setState(State::OpenChannel);
 }
@@ -1233,7 +1247,7 @@ void SshSession::authenticateWithPassword()
         return;
     }
 
-    logInfo("Successfully authenticated with password.");
+    logInfoWithInject("Successfully authenticated with password.");
 
     setState(State::OpenChannel);
 }
@@ -1288,7 +1302,8 @@ bool SshSession::authenticateWithAgent()
         }
         if (rc == LIBSSH2_ERROR_NONE)
         {
-            logInfo("Successfully authenticated with SSH agent with identity: {}", identity->comment);
+            logInfoWithInject("Successfully authenticated with SSH agent with identity: {}",
+                              identity->comment);
             setState(State::OpenChannel);
             return true;
         }
