@@ -5,13 +5,13 @@
 
 #include <crispy/logstore.h>
 
-#include <fmt/format.h>
-
 #include <range/v3/view/filter.hpp>
 #include <range/v3/view/iota.hpp>
 #include <range/v3/view/zip.hpp>
 
 #include <array>
+#include <cstdint>
+#include <format>
 
 using namespace std::string_view_literals;
 
@@ -66,7 +66,7 @@ namespace detail
     namespace
     {
 
-        enum class Thickness
+        enum class Thickness : uint8_t
         {
             Light,
             Heavy
@@ -129,7 +129,7 @@ namespace detail
             auto const w = unbox(imageSize.width);
             auto const h = unbox(imageSize.height);
 
-            // fmt::print("{}.drawArc: size={}\n", arc, imageSize);
+            // std::cout << std::format("{}.drawArc: size={}\n", arc, imageSize);
             auto const putpixel = [&](int x, int y, uint8_t alpha = 0xFFu) {
                 auto const fy = clamp((unsigned) y, 0u, h - 1);
                 auto const fx = clamp((unsigned) x, 0u, w - 1);
@@ -173,7 +173,7 @@ namespace detail
 
         struct ProgressBar
         {
-            enum class Part
+            enum class Part : uint8_t
             {
                 Left,
                 Middle,
@@ -595,7 +595,7 @@ namespace detail
 
         struct DiagonalMosaic
         {
-            enum class Body
+            enum class Body : uint8_t
             {
                 Lower,
                 Upper
@@ -729,7 +729,7 @@ namespace detail
             return pixmap.take();
         }
 
-        enum class UpperOrLower
+        enum class UpperOrLower : uint8_t
         {
             Upper,
             Lower
@@ -870,7 +870,7 @@ namespace detail
             auto const x1 = x0 + 1;
             auto const y1 = y0 + 1;
 
-            // fmt::print("- block sextant pos {}: x={} y={} x0={} y0={} x1={} y1={}\n",
+            // std::cout << std::format("- block sextant pos {}: x={} y={} x0={} y0={} x1={} y1={}\n",
             //            position, x, y, x0, y0, x1, y1);
 
             fillBlock(image, size, { x0 / 2_th, y0 / 3_th }, { x1 / 2_th, y1 / 3_th }, [](int, int) {
@@ -964,9 +964,10 @@ auto BoxDrawingRenderer::createTileData(char32_t codepoint, atlas::TileLocation
     {
         auto const supersamplingFactor = []() {
             auto constexpr EnvName = "SSA_FACTOR";
-            if (!getenv(EnvName))
+            auto* const envValue = getenv(EnvName);
+            if (!envValue)
                 return 2;
-            auto const val = atoi(getenv(EnvName));
+            auto const val = atoi(envValue);
             if (!(val >= 1 && val <= 8))
                 return 1;
             return val;
@@ -1042,38 +1043,20 @@ optional<atlas::Buffer> BoxDrawingRenderer::buildElements(char32_t codepoint)
     auto const ld = [=](Ratio a, Ratio b) {
         return lowerDiagonalMosaic(size, a, b);
     };
-    auto const lineArt =
-#if __cplusplus > 201703L
-        [=, this]
-#else
-        [=]
-#endif
-        () {
-            auto b = blockElement<2>(size);
-            b.getlineThickness(_gridMetrics.underline.thickness);
-            return b;
-        };
-    auto const progressBar =
-#if __cplusplus > 201703L
-        [=, this]
-#else
-        [=]
-#endif
-        () {
-            return ProgressBar { size, _gridMetrics.underline.position };
-        };
-    auto const segmentArt =
-#if __cplusplus > 201703L
-        [=, this]
-#else
-        [=]
-#endif
-        () {
-            auto constexpr AntiAliasingSamplingFactor = 1;
-            return blockElement<AntiAliasingSamplingFactor>(size)
-                .getlineThickness(_gridMetrics.underline.thickness)
-                .baseline(_gridMetrics.baseline * AntiAliasingSamplingFactor);
-        };
+    auto const lineArt = [size, this]() {
+        auto b = blockElement<2>(size);
+        b.getlineThickness(_gridMetrics.underline.thickness);
+        return b;
+    };
+    auto const progressBar = [size, this]() {
+        return ProgressBar { size, _gridMetrics.underline.position };
+    };
+    auto const segmentArt = [size, this]() {
+        auto constexpr AntiAliasingSamplingFactor = 1;
+        return blockElement<AntiAliasingSamplingFactor>(size)
+            .getlineThickness(_gridMetrics.underline.thickness)
+            .baseline(_gridMetrics.baseline * AntiAliasingSamplingFactor);
+    };
 
     // TODO: just check notcurses-info to get an idea what may be missing
     // clang-format off
