@@ -141,9 +141,6 @@ Making use of reserved glyph slots
 #include <libunicode/convert.h>
 #include <libunicode/utf8_grapheme_segmenter.h>
 
-#include <fmt/core.h>
-#include <fmt/ostream.h>
-
 #include <range/v3/algorithm/copy.hpp>
 #include <range/v3/view/enumerate.hpp>
 
@@ -258,28 +255,10 @@ text::font_locator& createFontLocator(FontLocatorEngine engine)
     switch (engine)
     {
         case FontLocatorEngine::Mock: return text::font_locator_provider::get().mock();
-        case FontLocatorEngine::DWrite:
-#if defined(_WIN32)
-            return text::font_locator_provider::get().directwrite();
-#else
-            locatorLog()("Font locator DirectWrite not supported on this platform.");
-#endif
-            break;
-        case FontLocatorEngine::CoreText:
-#if defined(__APPLE__)
-            return text::font_locator_provider::get().coretext();
-#else
-            locatorLog()("Font locator CoreText not supported on this platform.");
-#endif
-            break;
-
-        case FontLocatorEngine::FontConfig:
-            // default case below
-            break;
+        default: return text::font_locator_provider::get().native();
     }
 
-    locatorLog()("Using font locator: fontconfig.");
-    return text::font_locator_provider::get().fontconfig();
+    crispy::unreachable();
 }
 
 // TODO: What's a good value here? Or do we want to make that configurable,
@@ -426,7 +405,7 @@ Renderable::AtlasTileAttributes const* TextRenderer::ensureRasterizedIfDirectMap
     restrictToTileSize(*tileCreateData);
     Require(tileCreateData->bitmapSize.width <= textureAtlas().tileSize().width);
 
-    // fmt::print("Initialize direct mapping {} ({}) for {}; {}; {}\n",
+    // std::cout << std::format("Initialize direct mapping {} ({}) for {}; {}; {}\n",
     //            tileIndex,
     //            tileLocation,
     //            glyph,
@@ -460,7 +439,7 @@ void TextRenderer::renderLine(vtbackend::RenderLine const& renderLine)
 
 void TextRenderer::renderCell(vtbackend::RenderCell const& cell)
 {
-    // fmt::print("renderCell: {} {} {} {} {}\n",
+    // std::cout << std::format("renderCell: {} {} {} {} {}\n",
     //            cell.position,
     //            unicode::convert_to<char>(u32string_view(cell.codepoints)),
     //            _forceUpdateInitialPenPosition ? "forcedRestart" : "-",
@@ -508,7 +487,7 @@ point TextRenderer::applyGlyphPositionToPen(point pen,
                                            //- tileAttributes.bitmapSize.height.as<int>() // -> bitmap height
         ;
 
-    // fmt::print("pen! {} <- {}, gpos {}, glyph offset {}x+{}y, glyph height {} ({})\n",
+    // std::cout << std::format("pen! {} <- {}, gpos {}, glyph offset {}x+{}y, glyph height {} ({})\n",
     //            Point { x, y },
     //            pen,
     //            gpos,
@@ -582,7 +561,6 @@ void TextRenderer::renderTextGroup(std::u32string_view codepoints,
     text::shape_result const& glyphPositions =
         getOrCreateCachedGlyphPositions(hash, codepoints, clusters, style);
     crispy::point pen = _gridMetrics.mapBottomLeft(initialPenPosition);
-    auto const advanceX = unbox(_gridMetrics.cellSize.width);
 
     for (auto const& glyphPosition: glyphPositions)
     {
@@ -590,7 +568,7 @@ void TextRenderer::renderTextGroup(std::u32string_view codepoints,
         {
             auto const pen1 = applyGlyphPositionToPen(pen, *attributes, glyphPosition);
             renderRasterizedGlyph(pen1, color, *attributes);
-            pen.x += static_cast<decltype(pen.x)>(advanceX);
+            pen.x += unbox<decltype(pen.x)>(_gridMetrics.cellSize.width);
             continue;
         }
 
@@ -617,10 +595,14 @@ void TextRenderer::renderTextGroup(std::u32string_view codepoints,
 
         if (glyphPosition.advance.x)
         {
+
+            auto numberOfCellsToAdvance =
+                std::rint(glyphPosition.advance.x / unbox<double>(_gridMetrics.cellSize.width));
             // Only advance horizontally, as we're (guess what) a terminal. :-)
             // Only advance in fixed-width steps.
-            // Only advance iff there harfbuzz told us to.
-            pen.x += static_cast<decltype(pen.x)>(advanceX);
+            // Only advance if there harfbuzz told us to.
+            pen.x +=
+                static_cast<decltype(pen.x)>(numberOfCellsToAdvance * (unbox(_gridMetrics.cellSize.width)));
         }
     }
 }
@@ -766,7 +748,7 @@ auto TextRenderer::createRasterizedGlyph(atlas::TileLocation tileLocation,
         }
 
         // Assume colored (RGBA) bitmap glyphs to be emoji.
-        // At least on MacOS/X, the emoji font reports bad positioning, so we simply center them ourself.
+        // At least on macOS, the emoji font reports bad positioning, so we simply center them ourself.
         glyph.position.x = unbox<int>(emojiBoundingBox.width - glyph.bitmapSize.width) / 2;
         glyph.position.y = unbox<int>(emojiBoundingBox.height)
                            - max(0, unbox<int>(emojiBoundingBox.height - glyph.bitmapSize.height) / 2);
